# Copyright (C) 2020 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

# pylint: disable=too-many-branches

import os
import re

from sos import _sos as _
from sos.policies import Policy
from sos.policies.init_systems import InitSystem
from sos.policies.init_systems.systemd import SystemdInit
from sos.policies.runtimes.crio import CrioContainerRuntime
from sos.policies.runtimes.podman import PodmanContainerRuntime
from sos.policies.runtimes.docker import DockerContainerRuntime
from sos.policies.runtimes.lxd import LxdContainerRuntime

from sos.utilities import (shell_out, is_executable, bold,
                           sos_get_command_output)


OS_RELEASE = "/etc/os-release"
# Container environment variables for detecting if we're in a container
ENV_CONTAINER = 'container'
ENV_HOST_SYSROOT = 'HOST'


class LinuxPolicy(Policy):
    """This policy is meant to be an abc class that provides common
    implementations used in Linux distros"""
    vendor = "None"
    PATH = "/bin:/sbin:/usr/bin:/usr/sbin"
    init = None
    # the following will be used, in order, as part of check() to validate that
    # we are running on a particular distro
    os_release_file = ''
    os_release_name = ''
    os_release_id = ''
    _upload_url = None
    default_container_runtime = 'docker'
    _preferred_hash_name = None
    # collector-focused class attrs
    containerized = False
    container_image = None
    sos_path_strip = None
    sos_pkg_name = None
    sos_bin_path = '/usr/bin'
    sos_container_name = 'sos-collector-tmp'
    container_version_command = None
    container_authfile = None

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

        if sysroot:
            self.sysroot = sysroot
        else:
            self.sysroot = self._container_init() or '/'

        self.init_kernel_modules()

        if init is not None:
            self.init_system = init
        elif os.path.isdir("/run/systemd/system/"):
            self.init_system = SystemdInit(chroot=self.sysroot)
        else:
            self.init_system = InitSystem()

        self.runtimes = {}
        if self.probe_runtime:
            _crun = [
                PodmanContainerRuntime(policy=self),
                DockerContainerRuntime(policy=self),
                CrioContainerRuntime(policy=self),
                LxdContainerRuntime(policy=self),
            ]
            for runtime in _crun:
                if runtime.check_is_active():
                    self.runtimes[runtime.name] = runtime
                    if runtime.name == self.default_container_runtime:
                        self.runtimes['default'] = self.runtimes[runtime.name]
                    self.runtimes[runtime.name].load_container_info()

            if self.runtimes and 'default' not in self.runtimes:
                # still allow plugins to query a runtime present on the system
                # even if that is not the policy default one
                idx = list(self.runtimes.keys())
                self.runtimes['default'] = self.runtimes[idx[0]]

    @classmethod
    def set_forbidden_paths(cls):
        return [
            '/etc/passwd',
            '/etc/shadow'
        ]

    @classmethod
    def check(cls, remote=''):
        """
        This function is responsible for determining if the underlying system
        is supported by this policy.
        """
        def _check_release(content):
            _matches = [cls.os_release_name]
            if cls.os_release_id:
                _matches.append(cls.os_release_id)
            for line in content.splitlines():
                if line.startswith(('NAME=', 'ID=')):
                    _distro = line.split('=')[1:][0].strip("\"'")
                    if _distro in _matches:
                        return True
            return False

        if remote:
            return _check_release(remote)
        # use the os-specific file primarily
        # also check the symlink destination
        if (os.path.isfile(cls.os_release_file) and
            os.path.basename(cls.os_release_file)
                == os.path.basename(os.path.realpath(cls.os_release_file))):
            return True
        # next check os-release for a NAME or ID value we expect
        with open(OS_RELEASE, "r", encoding='utf-8') as f:
            return _check_release(f.read())

    def kernel_version(self):
        return self.release

    def host_name(self):
        return self.hostname

    def is_kernel_smp(self):
        return self.smp

    def get_arch(self):
        return self.machine

    def get_local_name(self):
        """Returns the name usd in the pre_work step"""
        return self.host_name()

    def sanitize_filename(self, name):
        return re.sub(r"[^-a-z,A-Z.0-9]", "", name)

    @classmethod
    def display_help(cls, section):
        if cls == LinuxPolicy:
            cls.display_self_help(section)
        else:
            section.set_title(f"{cls.os_release_name} Distribution Policy")
            cls.display_distro_help(section)

    @classmethod
    def display_self_help(cls, section):
        section.set_title("SoS Distribution Policies")
        section.add_text(
            'Distributions supported by SoS will each have a specific policy '
            'defined for them, to ensure proper operation of SoS on those '
            'systems.'
        )

    @classmethod
    def display_distro_help(cls, section):
        if cls.__doc__ and cls.__doc__ is not LinuxPolicy.__doc__:
            section.add_text(cls.__doc__)
        else:
            section.add_text(
                '\nDetailed help information for this policy is not available'
            )

        # instantiate the requested policy so we can report more interesting
        # information like $PATH and loaded presets
        _pol = cls(None, None, False)
        section.add_text(
            f"Default --upload location: {_pol._upload_url}"
        )
        section.add_text(
            f"Default container runtime: {_pol.default_container_runtime}",
            newline=False
        )
        section.add_text(
            f"$PATH used when running report: {_pol.PATH}",
            newline=False
        )

        refsec = section.add_section('Reference URLs')
        for url in cls.vendor_urls:
            refsec.add_text(f"{' ':>8}{url[0]:<30}{url[1]:<40}", newline=False)

        presec = section.add_section('Presets Available With This Policy\n')
        presec.add_text(
            bold(
                f"{' ':>8}{'Preset Name':<20}{'Description':<45}"
                f"{'Enabled Options':<30}"
            ),
            newline=False
        )
        for preset, value in _pol.presets.items():
            _opts = ' '.join(value.opts.to_args())
            presec.add_text(
                f"{' ':>8}{preset:<20}{value.desc:<45}{_opts:<30}",
                newline=False
            )

    def _container_init(self):
        """Check if sos is running in a container and perform container
        specific initialisation based on ENV_HOST_SYSROOT.
        """
        if ENV_CONTAINER in os.environ:
            if os.environ[ENV_CONTAINER] in ['docker', 'oci', 'podman']:
                self._in_container = True
                if ENV_HOST_SYSROOT in os.environ:
                    if not os.environ[ENV_HOST_SYSROOT]:
                        # guard against blank/improperly unset values
                        return None
                    self._tmp_dir = os.path.abspath(
                        os.environ[ENV_HOST_SYSROOT] + self._tmp_dir
                    )
                    return os.environ[ENV_HOST_SYSROOT]
        return None

    def init_kernel_modules(self):
        """Obtain a list of loaded kernel modules to reference later for plugin
        enablement and SoSPredicate checks
        """
        self.kernel_mods = []
        release = os.uname().release

        # first load modules from lsmod
        lines = shell_out("lsmod", timeout=0, chroot=self.sysroot).splitlines()
        self.kernel_mods.extend([
            line.split()[0].strip() for line in lines[1:]
        ])

        # next, include kernel builtins
        builtins = self.join_sysroot(
            f"/usr/lib/modules/{release}/modules.builtin"
        )
        try:
            with open(builtins, "r", encoding='utf-8') as mfile:
                for line in mfile:
                    kmod = line.split('/')[-1].split('.ko')[0]
                    self.kernel_mods.append(kmod)
        except IOError as err:
            self.soslog.warning(f"Unable to read kernel builtins: {err}")

        # finally, parse kconfig looking for specific kconfig strings that
        # have been verified to not appear in either lsmod or modules.builtin
        # regardless of how they are built
        config_strings = {
            'devlink': 'CONFIG_NET_DEVLINK',
            'dm_mod': 'CONFIG_BLK_DEV_DM'
        }

        kconfigs = (
            f"/boot/config-{release}",
            f"/lib/modules/{release}/config",
        )
        for kconfig in kconfigs:
            kconfig = self.join_sysroot(kconfig)
            if os.path.exists(kconfig):
                booted_config = kconfig
                break
        else:
            self.soslog.warning("Unable to find booted kernel config")
            return

        kconfigs = []
        try:
            with open(booted_config, "r", encoding='utf-8') as kfile:
                for line in kfile:
                    if '=y' in line:
                        kconfigs.append(line.split('=y')[0])
        except IOError as err:
            self.soslog.warning(f"Unable to read booted kernel config: {err}")

        for builtin, value in config_strings.items():
            if value in kconfigs:
                self.kernel_mods.append(builtin)

    def join_sysroot(self, path):
        if self.sysroot and self.sysroot != '/':
            path = os.path.join(self.sysroot, path.lstrip('/'))
        return path

    def pre_work(self):
        # this method will be called before the gathering begins

        cmdline_opts = self.commons['cmdlineopts']

        if cmdline_opts.low_priority:
            self._configure_low_priority()

        # set or query for case id
        self.case_id = self.prompt_for_case_id(cmdline_opts)

    def prompt_for_case_id(self, cmdline_opts):
        if not cmdline_opts.batch and not \
                cmdline_opts.quiet:
            if not cmdline_opts.case_id:
                cmdline_opts.case_id = input(
                    _("Optionally, please enter the case id that you are "
                      "generating this report for: ")
                )
        self.case_id = cmdline_opts.case_id if \
            cmdline_opts.case_id else ""

        return self.case_id

    def _configure_low_priority(self):
        """Used to constrain sos to a 'low priority' execution, potentially
        letting individual policies set their own definition of what that is.

        By default, this will attempt to assign sos to an idle io class via
        ionice if available. We will also renice our own pid to 19 in order to
        not cause competition with other host processes for CPU time.
        """
        _pid = os.getpid()
        if is_executable('ionice'):
            ret = sos_get_command_output(
                f"ionice -c3 -p {_pid}", timeout=5
            )
            if ret['status'] == 0:
                self.soslog.info('Set IO class to idle')
            else:
                msg = (f"Error setting IO class to idle: {ret['output']} "
                       f"(exit code {ret['status']})")
                self.soslog.error(msg)
        else:
            self.ui_log.warning(
                "Warning: unable to constrain report to idle IO class: "
                "ionice is not available."
            )

        try:
            os.nice(20)
            self.soslog.info('Set niceness of report to 19')
        except Exception as err:
            self.soslog.error(f"Error setting report niceness to 19: {err}")

    def set_sos_prefix(self):
        """If sos report commands need to always be prefixed with something,
        for example running in a specific container image, then it should be
        defined here.

        If no prefix should be set, return an empty string instead of None.
        """
        return ''

    def set_cleanup_cmd(self):
        """If a host requires additional cleanup, the command should be set and
        returned here
        """
        return ''

    # pylint: disable=unused-argument
    def create_sos_container(self, image=None, auth=None, force_pull=False):
        """Returns the command that will create the container that will be
        used for running commands inside a container on hosts that require it.

        This will use the container runtime defined for the host type to
        launch a container. From there, we use the defined runtime to exec into
        the container's namespace.

        :param image:   The name of the image if not using the policy default
        :type image:    ``str`` or ``None``

        :param auth:    The auth string required by the runtime to pull an
                        image from the registry
        :type auth:     ``str`` or ``None``

        :param force_pull:  Should the runtime forcibly pull the image
        :type force_pull:   ``bool``

        :returns:   The command to execute to launch the temp container
        :rtype:     ``str``
        """
        return ''

    def restart_sos_container(self):
        # pylint: disable=no-member
        """Restarts the container created for sos collect if it has stopped.

        This is called immediately after create_sos_container() as the command
        to create the container will exit and the container will stop. For
        current container runtimes, subsequently starting the container will
        default to opening a bash shell in the container to keep it running,
        thus allowing us to exec into it again.
        """
        return f"{self.container_runtime} start {self.sos_container_name}"

    def format_container_command(self, cmd):
        # pylint: disable=no-member
        """Returns the command that allows us to exec into the created
        container for sos collect.

        :param cmd: The command to run in the sos container
        :type cmd: ``str``

        :returns: The command to execute to run `cmd` in the container
        :rtype: ``str``
        """
        if self.container_runtime:
            return (f'{self.container_runtime} exec {self.sos_container_name} '
                    f'{cmd}')
        return cmd


class GenericLinuxPolicy(LinuxPolicy):
    """This Policy will be returned if no other policy can be loaded. This
    should allow for IndependentPlugins to be executed on any system"""

    vendor_urls = [('Upstream Project', 'https://github.com/sosreport/sos')]
    vendor = 'SoS'
    vendor_text = ('SoS was unable to determine that the distribution of this '
                   'system is supported, and has loaded a generic '
                   'configuration. This may not provide desired behavior, and '
                   'users are encouraged to request a new distribution-specifc'
                   ' policy at the GitHub project above.\n')

    @classmethod
    def check(cls, remote=''):
        """
        This function is responsible for determining if the underlying system
        is supported by this policy.
        """
        raise NotImplementedError

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.#

import os

from sos.report.plugins import Plugin, PluginOpt, UbuntuPlugin


class CharmedMySQL(Plugin, UbuntuPlugin):
    """
    The Charmed MySQL plugin is used to collect MySQL configuration and logs
    from the Charmed MySQL snap package. It also collects MySQL Router
    and MySQL Shell configuration and logs where available,
    journal logs for the snap, and snap info.

    If the `dumpdbs` option is set to `True`, the plugin will also try and
    collect the names of the databases that the user has access to. The
    `mysql` user is used by default, but that can be set using the `dbuser`
    option. When using the `dumpdbs` option, you must then provide the
    password for the user using the `dbpass` option.
    """

    short_desc = "Charmed MySQL"
    plugin_name = "charmed_mysql"
    packages = ("charmed-mysql",)

    option_list = [
        PluginOpt(
            "dbuser", default="mysql", val_type=str,
            desc="Username for database dump collection"
        ),
        PluginOpt(
            "dbpass", default="", val_type=str,
            desc="Password for database dump collection",
        ),
        PluginOpt(
            "dumpdbs", default=False, val_type=bool,
            desc="Get name of all databases"
        ),
    ]

    def setup(self):
        # Set default paths for snap
        snap_path_common = "/var/snap/charmed-mysql/common"
        snap_path_current = "/var/snap/charmed-mysql/current"

        # Check if deployment is in a Kubernetes cluster
        k8s_deploy = any("KUBERNETES" in key for key in os.environ)

        # If in a Kubernetes cluster, set the snap paths to "None" as
        # MySQL configuration does not exist in snap paths
        if k8s_deploy:
            snap_path_common = ""
            snap_path_current = ""

        # Set the configuration paths
        conf_paths = {
            "MYSQL_CONF": f"{snap_path_current}/etc/mysql",
            "MYSQL_LOGS": f"{snap_path_common}/var/log/mysql",
            "MYSQL_ROUTER_CONF": f"{snap_path_current}/etc/mysqlrouter",
            "MYSQL_ROUTER_LOGS": f"{snap_path_common}/var/log/mysqlrouter",
            "MYSQL_SHELL_LOGS": f"{snap_path_common}/var/log/mysqlsh",
        }

        # Ignore private keys
        self.add_forbidden_path([
            f"{conf_paths['MYSQL_CONF']}/*.pem",
            f"{conf_paths['MYSQL_CONF']}/*.key",
        ])

        # Include the files we want to get
        self.add_copy_spec([
            conf_paths["MYSQL_CONF"],
            conf_paths["MYSQL_LOGS"],
            conf_paths["MYSQL_ROUTER_CONF"],
            conf_paths["MYSQL_ROUTER_LOGS"],
            conf_paths["MYSQL_SHELL_LOGS"],
        ])

        # Only applicable if not in a Kubernetes cluster
        # as `snap info` and `journalctl` are not available
        # in the Kubernetes container
        if not k8s_deploy:
            # Get snap logs
            self.add_journal("snap.charmed-mysql.*")

            # Get snap info
            self.add_cmd_output("snap info charmed-mysql")

        # If dumpdbs is set, then get all databases
        if self.get_option("dumpdbs"):

            db_user = self.get_option("dbuser")
            db_pass = self.get_option("dbpass")

            # Check password is not already an environment variable
            # and user did not supply a password
            if "MYSQL_PWD" in os.environ and not db_pass:
                self.soslog.info(
                    "MYSQL_PWD present: Using MYSQL_PWD environment variable, "
                    "user did not provide password."
                )
                db_pass = os.environ["MYSQL_PWD"]
            elif not db_pass:  # Environment variable not set and no password
                self.soslog.warning(
                    "dumpdbs_error: option is set, but username and password "
                    "are not provided"
                )
                return

            mysql_env = {"MYSQL_PWD": db_pass}
            opts = f"-h 127.0.0.1 -u{db_user}"
            sql_cmd = f"mysql {opts} -e"
            queries = [
                # Get databases user has access to
                "'show databases;'",

                # Get unit operations from MySQL to see, for example,
                # if a unit is stuck on joining the cluster
                "'select * from mysql.juju_units_operations;'",

                # Get the cluster group replication status
                ("'select * from performance_schema.replication_group_members "
                    "order by MEMBER_HOST;'"),

                # Get connection stats
                "'show global status like \"%conne%\";'",

                # Get errors per client and host
                # Useful for problens like an app disconnectting randomly
                "'select * from performance_schema.host_cache;'"
            ]

            self.add_cmd_output(
                [f"{sql_cmd} {query}" for query in queries],
                env=mysql_env
            )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class OpenStackDesignate(Plugin):

    short_desc = 'Openstack Designate'

    plugin_name = "openstack_designate"
    profiles = ('openstack', 'openstack_controller')

    var_puppet_gen = "/var/lib/config-data/puppet-generated/designate"
    var_ansible_gen = "/var/lib/config-data/ansible-generated"

    def setup(self):
        # collect current pool config

        self.add_cmd_output(
            "designate-manage pool generate_file --file /dev/stdout",
            container=self.get_container_by_name(".*designate_central"),
            suggest_filename="openstack_designate_current_pools.yaml"
        )

        # configs
        self.add_copy_spec([
            "/etc/designate/*",
            self.var_puppet_gen + "/etc/designate/designate.conf",
            self.var_puppet_gen + "/etc/designate/pools.yaml",
            self.var_ansible_gen + "/designate/etc/named.conf",
            self.var_ansible_gen + "/designate/etc/named/*",
            self.var_ansible_gen + "/unbound/*"
        ])

        # logs
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/designate/*",
                "/var/log/containers/designate/*",
                "/var/log/containers/designate-bind/*",
                "/var/log/containers/unbound/*"
            ])
        else:
            self.add_copy_spec([
                "/var/log/designate/*.log",
                "/var/log/containers/designate/*.log",
                "/var/log/containers/designate-bind/*.log",
                "/var/log/containers/unbound/*.log"
            ])

        subcmds = [
            'dns service list',
            'dns quota list',
            'ptr record list',
            'tld list',
            'tsigkey list --column name --column algorithm --column scope',
            'zone blacklist list',
            'zone export list',
            'zone import list',
            'zone list',
            'zone transfer accept list',
            'zone transfer request list'
        ]

        # commands
        self.add_cmd_output([
            f'openstack {sub} --all-projects' for sub in subcmds
        ])

        # get recordsets for each zone
        cmd = "openstack zone list -f value -c id"
        ret = self.exec_cmd(cmd)
        if ret['status'] == 0:
            for zone in ret['output'].splitlines():
                zone = zone.split()[0]
                self.add_cmd_output(
                    f"openstack recordset list --all-projects {zone}",
                    subdir='recordset')

    def postproc(self):
        protect_keys = [
            "password", "connection", "transport_url", "admin_password",
            "ssl_key_password", "ssl_client_key_password",
            "memcache_secret_key"
        ]
        regexp = fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)"

        self.do_path_regex_sub("/etc/designate/*", regexp, r"\1*********")
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/designate/*",
            regexp, r"\1*********"
        )


class RedHatdesignate(OpenStackDesignate, RedHatPlugin):

    packages = ('openstack-selinux',)


class Ubuntudesignate(OpenStackDesignate, UbuntuPlugin):

    packages = ('designate-common',)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import glob
from sos.report.plugins import Plugin, RedHatPlugin


class GlusterBlock(Plugin, RedHatPlugin):

    short_desc = 'Gluster Block'

    plugin_name = 'gluster_block'
    profiles = ('storage',)
    packages = ("gluster-block",)
    files = ("/usr/sbin/gluster-block",)

    def setup(self):

        # collect logs - apply log_size for any individual file
        # all_logs takes precedence over logsize
        if not self.get_option("all_logs"):
            limit = self.get_option("log_size")
        else:
            limit = 0

        if limit:
            for file in glob.glob("/var/log/gluster-block/*.log"):
                self.add_copy_spec(file, limit)
        else:
            self.add_copy_spec("/var/log/gluster-block")



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
import fnmatch

from sos.utilities import sos_get_command_output


class PackageManager():
    """Encapsulates a package manager. If you provide a query_command to the
    constructor it should print each package on the system in the following
    format::

        package name|package.version

    You may also subclass this class and provide a _generate_pkg_list method to
    build the list of packages and versions.

    :cvar query_command: The command to use for querying packages
    :vartype query_command: ``str`` or ``None``

    :cvar verify_command: The command to use for verifying packages
    :vartype verify_command: ``str`` or ``None``

    :cvar verify_filter: Optional filter to use for controlling package
                         verification
    :vartype verify_filter: ``str or ``None``

    :cvar files_command: The command to use for getting file lists for packages
    :vartype files_command: ``str`` or ``None``

    :cvar chroot: Perform a chroot when executing `files_command`
    :vartype chroot: ``bool``

    :cvar remote_exec: If package manager is on a remote system (e.g. for
                       sos collect), use this to execute commands
    :vartype remote_exec: ``SoSTransport.run_command()``
    """

    query_command = None
    verify_command = None
    verify_filter = None
    files_command = None
    query_path_command = None
    chroot = None
    files = None

    def __init__(self, chroot=None, remote_exec=None):
        self._packages = None
        self.files = []
        self.remote_exec = remote_exec

        if chroot:
            self.chroot = chroot

    @property
    def packages(self):
        if self._packages is None:
            self._generate_pkg_list()
        return self._packages

    @property
    def manager_name(self):
        return self.__class__.__name__.lower().split('package', maxsplit=1)[0]

    def exec_cmd(self, command, timeout=30, need_root=False, env=None,
                 use_shell=False, chroot=None):
        """
        Runs a package manager command, either via sos_get_command_output() if
        local, or via a SoSTransport's run_command() if this needs to be run
        remotely, as in the case of remote nodes for use during `sos collect`.

        :param command:     The command to execute
        :type command:      ``str``

        :param timeout:     Timeout for command to run, in seconds
        :type timeout:      ``int``

        :param need_root:   Does the command require root privileges?
        :type need_root:    ``bool``

        :param env:         Environment variables to set
        :type env:          ``dict`` with keys being env vars to define

        :param use_shell:   If running remotely, does the command require
                            obtaining a shell?
        :type use_shell:      ``bool``

        :param chroot:      If necessary, chroot command execution to here
        :type chroot:       ``None`` or ``str``

        :returns:   The output of the command
        :rtype:     ``str``
        """
        if self.remote_exec:
            ret = self.remote_exec(command, timeout, need_root, env, use_shell)
        else:
            ret = sos_get_command_output(command, timeout, chroot=chroot,
                                         env=env)
        if ret['status'] == 0:
            return ret['output']
        # In the case of package managers, we don't want to potentially iterate
        # over stderr, so prevent the package methods from doing anything at
        # all by returning nothing.
        return ''

    def all_pkgs_by_name(self, name):
        """
        Get a list of packages that match name.

        :param name: The name of the package
        :type name: ``str``

        :returns: List of all packages matching `name`
        :rtype: ``list``
        """
        return fnmatch.filter(self.packages.keys(), name)

    def all_pkgs_by_name_regex(self, regex_name, flags=0):
        """
        Get a list of packages that match regex_name.

        :param regex_name: The regex to use for matching package names against
        :type regex_name: ``str``

        :param flags: Flags for the `re` module when matching `regex_name`

        :returns: All packages matching `regex_name`
        :rtype: ``list``
        """
        reg = re.compile(regex_name, flags)
        return [pkg for pkg in self.packages if reg.match(pkg)]

    def pkg_by_name(self, name):
        """
        Get a single package that matches name.

        :param name: The name of the package
        :type name: ``str``

        :returns: The first package that matches `name`
        :rtype: ``str``
        """
        try:
            return self.packages[name]
        except Exception:
            return None

    def _parse_pkg_list(self, pkg_list):
        """
        Using the output of `query_command`, build the _packages dict.

        This should be overridden by distinct package managers and be a
        generator for _generate_pkg_list which will insert the packages into
        the _packages dict.

        This method should yield a tuple of name, version, release for each
        package parsed. If the package manager or distribution does not use a
        release field, set it to None.

        :param pkg_list: The output of the result of `query_command`
        :type pkg_list:  ``str``
        """
        raise NotImplementedError

    def _generate_pkg_list(self):
        """Generates a dictionary of packages for internal use by the package
        manager in the format::

            {'package_name': {'name': 'package_name',
                              'version': 'major.minor.version',
                              'release': 'package release' or None,
                              'pkg_manager': 'package manager name'}}

        """
        if self._packages is None:
            self._packages = {}

        if self.query_command:
            cmd = self.query_command
            pkg_list = self.exec_cmd(cmd, timeout=30, chroot=self.chroot)

            for pkg in self._parse_pkg_list(pkg_list):
                self._packages[pkg[0]] = {
                    'name': pkg[0],
                    'version': pkg[1].split('.'),
                    'release': pkg[2],
                    'pkg_manager': self.manager_name
                }

    def pkg_version(self, pkg):
        """Returns the entry in self.packages for pkg if it exists

        :param pkg: The name of the package
        :type pkg: ``str``

        :returns: Package name and version, if package exists
        :rtype: ``dict`` if found, else ``None``
        """
        if pkg in self.packages:
            return self.packages[pkg]
        return None

    def all_files(self):
        """
        Get a list of files known by the package manager

        :returns: All files known by the package manager
        :rtype: ``list``
        """
        if self.files_command and not self.files:
            cmd = self.files_command
            files = self.exec_cmd(cmd, timeout=180, chroot=self.chroot)
            self.files = files.splitlines()
        return self.files

    def pkg_by_path(self, path):
        """Given a path, return the package that owns that path.

        :param path:    The filepath to check for package ownership
        :type path:     ``str``

        :returns:       The package name or 'unknown'
        :rtype:         ``str``
        """
        if not self.query_path_command:
            return 'unknown'
        try:
            cmd = f"{self.query_path_command} {path}"
            pkg = self.exec_cmd(cmd, timeout=5, chroot=self.chroot)
            return pkg.splitlines() or 'unknown'
        except Exception:
            return 'unknown'

    def build_verify_command(self, packages):
        """build_verify_command(self, packages) -> str
            Generate a command to verify the list of packages given
            in ``packages`` using the native package manager's
            verification tool.

            The command to be executed is returned as a string that
            may be passed to a command execution routine (for e.g.
            ``sos_get_command_output()``.

            :param packages: a string, or a list of strings giving
                             package names to be verified.
            :returns: a string containing an executable command
                      that will perform verification of the given
                      packages.
            :rtype: str or ``NoneType``
        """
        if not self.verify_command:
            return None

        # The re.match(pkg) used by all_pkgs_by_name_regex() may return
        # an empty list (`[[]]`) when no package matches: avoid building
        # an rpm -V command line with the empty string as the package
        # list in this case.
        by_regex = self.all_pkgs_by_name_regex
        verify_list = filter(None, map(by_regex, packages))

        # No packages after regex match?
        if not verify_list:
            return None

        verify_packages = ""
        for package_list in verify_list:
            for package in package_list:
                if any(f in package for f in self.verify_filter):
                    continue
                if verify_packages:
                    verify_packages += " "
                verify_packages += package
        return self.verify_command + " " + verify_packages


class MultiPackageManager(PackageManager):
    """
    This class is used to leverage multiple individual package managers as a
    single entity on systems that support multiple concurrent package managers.

    Policies that use this approach will need to specify a primary package
    manager, and at least one fallback manager. When queries are sent to this
    manager, the primary child manager is checked first. If there is a valid,
    not None, response (e.g. a given package is installed) then that response
    is used. However, if the response is empty or None, the fallback managers
    are then queried in the order they were passed to MultiPackageManager
    during initialization.

    :param primary: The primary package manager to rely on
    :type primary:  A subclass of `PackageManager`

    :param fallbacks: A list of package managers to use if the primary does not
                      provide a response
    :type fallbacks: ``list`` of `PackageManager` subclasses
    """

    def __init__(self, primary, fallbacks, chroot=None, remote_exec=None):
        super().__init__(chroot=chroot, remote_exec=remote_exec)

        if not issubclass(primary, PackageManager):
            raise Exception(
                f"Primary package manager must be PackageManager subclass, not"
                f" {primary.__class__}"
            )

        if not isinstance(fallbacks, list):
            raise Exception('Fallbacks must be specified in a list')

        for pm in fallbacks:
            if not issubclass(pm, PackageManager):
                raise Exception(
                    f"Fallback package managers must be PackageManager "
                    f"subclass, not {pm.__class__}"
                )

        self.primary = primary(chroot=chroot, remote_exec=remote_exec)
        self.fallbacks = [
            pm(chroot=chroot, remote_exec=remote_exec) for pm in fallbacks
        ]

        if not self.fallbacks:
            raise Exception(
                'Must define at least one fallback package manager'
            )

        self._managers = [self.primary]
        self._managers.extend(self.fallbacks)

    def _parse_pkg_list(self, pkg_list):
        """
        Using the output of `query_command`, build the _packages dict.

        This should be overridden by distinct package managers and be a
        generator for _generate_pkg_list which will insert the packages into
        the _packages dict.

        This method should yield a tuple of name, version, release for each
        package parsed. If the package manager or distribution does not use a
        release field, set it to None.

        :param pkg_list: The output of the result of `query_command`
        :type pkg_list:  ``str``
        """
        raise NotImplementedError

    def all_files(self):
        if not self.files:
            for pm in self._managers:
                self.files.extend(pm.all_files())
        return self.files

    def _generate_pkg_list(self):
        if self._packages is None:
            self._packages = {}

        self._packages.update(self.primary.packages)
        for pm in self.fallbacks:
            _pkgs = pm.packages
            for pkg in _pkgs.keys():
                if pkg not in self._packages:
                    self._packages[pkg] = _pkgs[pkg]

    def _pm_wrapper(self, method):
        """
        This wrapper method is used to provide implicit iteration through the
        primary and any defined fallback managers that are set for a given
        instance of MultiPackageManager.

        Important note: we pass the _name_ of the method to run here as a
        string, and not any actual method as we rely on iteratively looking up
        the actual method in each package manager.

        :param method: The name of the method we're wrapping for the purpose of
                       iterating through defined package managers
        :type method:  ``str``
        """
        def pkg_func(*args, **kwargs):
            ret = None
            for pm in self._managers:
                if not ret or ret == 'unknown':
                    _wrapped_func = getattr(pm, method)
                    ret = _wrapped_func(*args, **kwargs)
            return ret
        return pkg_func

    def __getattribute__(self, item):
        # if the attr is callable, then we need to iterate over our child
        # managers until we get a response, unless it is _generate_pkg_list in
        # which case we only want to use the one actually defined here, or
        # _pm_wrapper, which we need to avoid this override for to not hit
        # recursion hell.
        if item in ['_generate_pkg_list', '_pm_wrapper', 'all_files']:
            return super().__getattribute__(item)
        attr = super().__getattribute__(item)
        if hasattr(attr, '__call__'):
            return self._pm_wrapper(item)
        return attr

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)


class Apache(Plugin):
    """The Apache plugin covers the upstream Apache webserver project,
    regardless of the packaged name; apache2 for Debian and Ubuntu, or httpd
    for Red Hat family distributions.

    The aim of this plugin is for Apache-specific information, not necessarily
    other projects that happen to place logs or similar files within the
    standardized apache directories. For example, OpenStack components that log
    to apache logging directories are excluded from this plugin and collected
    via their respective OpenStack plugins.

    Users can expect the collection of apachectl command output, apache server
    logs, and apache configuration files from this plugin.
    """

    short_desc = 'Apache http daemon'
    plugin_name = "apache"
    profiles = ('webserver', 'openshift')
    packages = ('httpd',)
    files = ('/var/www/',)
    apachepkg = None

    option_list = [
        PluginOpt(name="log", default=False, desc="gathers all apache logs")
    ]

    def setup(self):
        # collect list of installed modules and verify config syntax.
        self.add_cmd_output([
            "apachectl -S",
            "apachectl -t"
        ], cmd_as_tag=True)
        self.add_cmd_output("apachectl -M", tags="httpd_M")

        # Other plugins collect these files;
        # do not collect them here to avoid collisions in the archive paths.
        subdirs = [
            'aodh',
            'ceilometer',
            'cinder',
            'foreman',
            'gnocchi',
            'horizon',
            'keystone',
            'manila',
            'nova',
            'octavia',
            'placement',
            'pulp'
        ]
        self.add_forbidden_path([
            f"/var/log/{self.apachepkg}*/{sub}*" for sub in subdirs
        ])


class RedHatApache(Apache, RedHatPlugin):
    """
    On Red Hat distributions, the Apache plugin will also attempt to collect
    JBoss Web Server logs and configuration files.

    Note that for Red Hat distributions, this plugin explicitly collects for
    'httpd' installations. If you have installed apache from source or via any
    method that uses the name 'apache' instead of 'httpd', these collections
    will fail.
    """
    files = (
        '/etc/httpd/conf/httpd.conf',
        '/etc/httpd22/conf/httpd.conf',
        '/etc/httpd24/conf/httpd.conf'
    )
    apachepkg = 'httpd'

    def setup(self):

        self.add_file_tags({
            "/var/log/httpd/access_log": 'httpd_access_log',
            "/var/log/httpd/error_log": 'httpd_error_log',
            "/var/log/httpd/ssl_access_log": 'httpd_ssl_access_log',
            "/var/log/httpd/ssl_error_log": 'httpd_ssl_error_log'
        })

        super().setup()

        # httpd versions, including those used for JBoss Web Server
        vers = ['', '22', '24']

        # Extrapolate all top-level config directories for each version, and
        # relevant config files within each
        etcdirs = [f"/etc/httpd{ver}" for ver in vers]
        confs = [
            "conf/*.conf",
            "conf.d/*.conf",
            "conf.modules.d/*.conf"
        ]

        # Extrapolate top-level logging directories for each version, and the
        # relevant log files within each
        logdirs = [f"/var/log/httpd{ver}" for ver in vers]
        logs = [
            "access_log",
            "error_log",
            "ssl_access_log",
            "ssl_error_log"
        ]

        self.add_forbidden_path([
            f"{etc}/conf/password.conf" for etc in etcdirs
        ])

        for edir in etcdirs:
            for conf in confs:
                self.add_copy_spec(f"{edir}/{conf}", tags="httpd_conf")

        if self.get_option("log") or self.get_option("all_logs"):
            self.add_copy_spec(logdirs)
        else:
            for ldir in logdirs:
                for log in logs:
                    self.add_copy_spec(f"{ldir}/{log}")

        self.add_copy_spec([
            "/var/lib/httpd/*",
        ])

        self.add_forbidden_path([
            "/var/lib/httpd/*.pem",
        ])

        self.add_service_status('httpd', tags='systemctl_httpd')

    def postproc(self):
        # Example for scrubbing RSA keys from json files
        #
        #     "key": {
        #       "kty": "RSA",
        #       "n": "<keydata>",
        #       "e": "<keydata>"
        #     }
        #
        # to
        #
        #     "key": {
        #       "kty": "RSA",
        #      "n": ********,
        #      "e": ********
        #     }
        self.do_path_regex_sub(
            '/var/lib/httpd/.*json',
            r"(\"e\"\s*:|\"n\"\s*:)(.*[,]$)",
            r"\1 ********,"
        )
        self.do_path_regex_sub(
            '/var/lib/httpd/.*json',
            r"(\"e\"\s*:|\"n\"\s*:)(.*[^,]$)",
            r"\1 ********"
        )


class DebianApache(Apache, DebianPlugin, UbuntuPlugin):
    files = ('/etc/apache2/apache2.conf',)
    apachepkg = 'apache2'

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/apache2/*",
            "/etc/default/apache2"
        ])

        self.add_service_status('apache2')

        # collect only the current log set by default
        self.add_copy_spec([
            "/var/log/apache2/access.log",
            "/var/log/apache2/error.log",
            "/var/log/apache2/ssl_access.log",
            "/var/log/apache2/ssl_error.log",
            "/var/log/apache2/other_vhosts_access.log",
        ])
        if self.get_option("log") or self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/apache2",
            ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc., Lee Yarwood <lyarwood@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackPlacement(Plugin):

    short_desc = 'OpenStack Placement'
    plugin_name = "openstack_placement"
    profiles = ('openstack', 'openstack_controller')
    containers = ('.*placement_api',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/placement"
    service_name = 'openstack-placement-api'
    apachepkg = None

    def setup(self):

        # collect commands output only if the openstack-placement-api service
        # is running

        in_container = self.container_exists('.*placement_api')

        if self.is_service_running(self.service_name) or in_container:
            placement_config = ""
            # if containerized we need to pass the config to the cont.
            if in_container:
                placement_config = "--config-dir " + self.var_puppet_gen + \
                                "/etc/placement/"
            self.add_cmd_output(
                "placement-manage " + placement_config + " db version",
                suggest_filename="placement-manage_db_version"
            )

            vars_all = [p in os.environ for p in [
                        'OS_USERNAME', 'OS_PASSWORD']]

            vars_any = [p in os.environ for p in [
                        'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

            if not (all(vars_all) and any(vars_any)):
                self.soslog.warning("Not all environment variables set. "
                                    "Source the environment file for the user "
                                    "intended to connect to the OpenStack "
                                    "environment.")
            else:
                res = self.collect_cmd_output(
                    "openstack resource provider list"
                )

                if res['status'] == 0:
                    resource_provider_list = res['output']
                    for provider in resource_provider_list.splitlines()[3:-1]:
                        res_provider = provider.split()[1]
                        sub_cmds = [
                            "inventory",
                            "trait",
                            "aggregate",
                        ]
                        self.add_cmd_output([
                            f"openstack resource provider {sub_cmd} list "
                            f"{res_provider}"
                            for sub_cmd in sub_cmds
                        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/placement/",
                "/var/log/containers/placement/",
                "/var/log/containers/httpd/placement-api/",
                f"/var/log/{self.apachepkg}*/placement*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/placement/*.log",
                "/var/log/containers/placement/*.log",
                "/var/log/containers/httpd/placement-api/*log",
                f"/var/log/{self.apachepkg}*/placement*.log",
            ])

        self.add_copy_spec([
            "/etc/placement/",
            self.var_puppet_gen + "/etc/placement/",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf",
            self.var_puppet_gen + "/etc/httpd/conf/",
            self.var_puppet_gen + "/etc/httpd/conf.d/",
            self.var_puppet_gen + "/etc/httpd/conf.modules.d/*.conf",
        ])

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/placement/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/placement/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "password",
            "memcache_secret_key",
            "NOVA_API_PASS",
            "PLACEMENT_PASS",
        ]
        connection_keys = [
            "database_connection",
            "slave_connection",
            "connection",
        ]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianPlacement(OpenStackPlacement, DebianPlugin, UbuntuPlugin):

    apachepkg = "apache2"
    packages = (
        'placement-common',
        'placement-api',
        'python3-placement',
    )


class RedHatPlacement(OpenStackPlacement, RedHatPlugin):

    apachepkg = "httpd"
    packages = ('openstack-selinux',)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Opensvc(Plugin, IndependentPlugin):

    short_desc = 'OpenSVC cluster and services (config and state collection)'
    plugin_name = 'opensvc'
    profiles = ('cluster', 'services', 'system')
    packages = ('opensvc',)

    def get_status(self, kind):
        """ Get the status of opensvc management service """
        getobjs = self.collect_cmd_output(f"om {kind} ls --color=no")
        dirname = kind + '_status'
        if getobjs['status'] == 0:
            for line in getobjs['output'].splitlines():
                self.add_cmd_output(
                    f"om {line} print status --color=no",
                    subdir=dirname
                )

    def setup(self):
        self.add_copy_spec([
            "/etc/opensvc/*",
            "/var/log/opensvc/*",
            "/etc/conf.d/opensvc",
            "/etc/default/opensvc",
            "/etc/sysconfig/opensvc",
            "/var/lib/opensvc/*.json",
            "/var/lib/opensvc/list.*",
            "/var/lib/opensvc/ccfg",
            "/var/lib/opensvc/cfg",
            "/var/lib/opensvc/certs/ca_certificates",
            "/var/lib/opensvc/certs/certificate_chain",
            "/var/lib/opensvc/compliance/*",
            "/var/lib/opensvc/namespaces/*",
            "/var/lib/opensvc/node/*",
            "/var/lib/opensvc/sec/*",
            "/var/lib/opensvc/svc/*",
            "/var/lib/opensvc/usr/*",
            "/var/lib/opensvc/vol/*",
        ])
        self.add_cmd_output([
            "om pool status --verbose --color=no",
            "om net status --verbose --color=no",
            "om mon --color=no",
            "om daemon dns dump --color=no",
            "om daemon relay status --color=no",
            "om daemon status --format flat_json --color=no"
        ])
        self.add_dir_listing('/var/lib/opensvc', recursive=True)
        self.get_status('vol')
        self.get_status('svc')
        pid_file = "/var/lib/opensvc/osvcd.pid"
        try:
            with open(pid_file, 'r', encoding='utf-8') as file:
                pid = file.read().strip()
                if not pid:
                    self._log_debug(f"{pid_file} is empty")
                    return
                if not pid.isdigit():
                    self._log_debug(f"Invalid PID in {pid_file}: {pid}")
                    return
                self.add_copy_spec(f"/proc/{pid}/task/*/status")
        except (IOError, FileNotFoundError, PermissionError) as error:
            self._log_debug(
                f"Error while reading PID file {pid_file}: {error}"
            )

    def postproc(self):
        # Example:
        #
        # [hb#2]
        # secret = mypassword
        # type = relay
        # timeout = 30
        #
        # to
        #
        # [hb#2]
        # secret = ****************************
        # type = relay
        # timeout = 30

        regexp = r"(\s*secret =\s*)\S+"
        self.do_file_sub(
            "/etc/opensvc/cluster.conf",
            regexp,
            r"\1****************************"
        )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class Alternatives(Plugin):

    short_desc = 'System alternatives'
    plugin_name = 'alternatives'
    alternatives_cmd = None
    alternatives_list = None

    def setup(self):

        self.add_cmd_output(f'{self.alternatives_cmd} --version')

        alts = []
        ignore = [
            'cdrecord',
            'ld',
            'mkisofs',
            'whois',
            'xinputrc'
        ]

        res = self.collect_cmd_output(self.alternatives_list)
        if res['status'] == 0:
            for line in res['output'].splitlines():
                alt = line.split()[0]
                if alt not in ignore:
                    alts.append(alt)
            disp_cmd = f"{self.alternatives_cmd} --display %s"
            self.add_cmd_output([disp_cmd % alt for alt in alts])


class RedHatAlternatives(Alternatives, RedHatPlugin):

    packages = ('alternatives',)
    commands = ('alternatives',)

    alternatives_cmd = 'alternatives'
    alternatives_list = f'{alternatives_cmd} --list'

    def setup(self):

        super().setup()

        self.add_cmd_tags({
            "alternatives --display java.*": 'display_java',
            "alternatives --display python.*":
                'alternatives_display_python'
        })


class UbuntuAlternatives(Alternatives, UbuntuPlugin):

    packages = ('dpkg',)
    commands = ('update-alternatives',)

    alternatives_cmd = 'update-alternatives'
    alternatives_list = f'{alternatives_cmd} --get-selections'

    def setup(self):

        super().setup()

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/alternatives.log*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/alternatives.log",
                "/var/log/alternatives.log.1",
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Chrony(Plugin):

    short_desc = 'Chrony clock (for Network time protocol)'

    plugin_name = "chrony"
    profiles = ('system', 'services')

    packages = ('chrony',)

    def setup(self):
        self.add_cmd_output([
            "chronyc activity",
            "chronyc tracking",
            "chronyc sourcestats",
            "chronyc serverstats",
            "chronyc ntpdata",
            "chronyc -n clients",
            "chronyc -N authdata",
            "chronyc -n selectdata",
        ])
        self.add_cmd_output("chronyc -n sources", tags="chronyc_sources")

        if self.get_option("all_logs"):
            self.add_copy_spec([
                    "/var/log/chrony/*",
            ])
        else:
            self.add_copy_spec([
                    "/var/log/chrony/*.log",
            ])


class RedHatChrony(Chrony, RedHatPlugin):
    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/chrony.conf",
            "/var/lib/chrony/drift"
        ])
        self.add_journal(units="chronyd")


class DebianChrony(Chrony, DebianPlugin, UbuntuPlugin):
    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/chrony/chrony.conf",
            "/etc/chrony/conf.d",
            "/etc/chrony/sources.d",
            "/var/lib/chrony/chrony.drift",
            "/etc/default/chrony"
        ])
        self.add_journal(units="chrony")

# vim: et ts=4 sw=4



# Copyright (C) 2017 Red Hat, Inc., Marcus Linden <mlinden@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, IndependentPlugin, SoSPredicate,
                                PluginOpt)


class Conntrack(Plugin, IndependentPlugin):

    short_desc = 'conntrack - netfilter connection tracking'

    plugin_name = 'conntrack'
    profiles = ('network', 'cluster')
    packages = ('conntrack-tools', 'conntrack', 'conntrackd')

    option_list = [
        PluginOpt("namespaces", default=None, val_type=int,
                  desc="Number of namespaces to collect, 0 for unlimited"),
    ]

    def setup(self):
        # Collect info from conntrackd
        self.add_copy_spec("/etc/conntrackd/conntrackd.conf")
        self.add_cmd_output([
            "conntrackd -s network",
            "conntrackd -s cache",
            "conntrackd -s runtime",
            "conntrackd -s link",
            "conntrackd -s rsqueue",
            "conntrackd -s queue",
            "conntrackd -s ct",
            "conntrackd -s expect",
        ])

        # Collect info from conntrack, only when all required kmods are loaded
        ct_pred = SoSPredicate(self,
                               kmods=['nf_conntrack',
                                      'nf_conntrack_netlink',
                                      'nf_defrag_ipv4',
                                      'nf_defrag_ipv6',
                                      'nfnetlink'],
                               required={'kmods': 'all'})
        self.add_cmd_output([
            "conntrack -L -o extended",
            "conntrack -S",
        ], pred=ct_pred)

        # Capture additional data from namespaces; each command is run
        # per-namespace
        cmd_prefix = "ip netns exec "
        nsps = self.get_option('namespaces')
        for namespace in self.get_network_namespaces(ns_max=nsps):
            ns_cmd_prefix = cmd_prefix + namespace + " "
            self.add_cmd_output(ns_cmd_prefix + "conntrack -L -o extended")
            self.add_cmd_output(ns_cmd_prefix + "conntrack -S")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc. Neependra Khare <nkhare@redhat.com>
# Copyright (C) 2015 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Etcd(Plugin, RedHatPlugin):
    """The etcd plugin collects information from the etcd key-value store. It
    is primarily used by Kubernetes/OpenShift clusters and is often run inside
    a container within the cluster.

    Collections will default to executing within an `etcdctl` container, if one
    is present, and only execute on the host if such a container is not
    currently running. The `etcdctl` name preference is adopted from OpenShift
    Container Platform deployments.

    This plugin is written for etcd v3 and later.
    """

    short_desc = 'etcd plugin'

    plugin_name = 'etcd'
    packages = ('etcd',)
    profiles = ('container', 'system', 'services', 'cluster')
    files = ('/etc/etcd/etcd.conf',)
    containers = ('etcdctl', 'etcd')
    services = ('etcd',)

    def setup(self):

        etcd_con = None
        for con in self.containers:
            if self.get_container_by_name(con):
                etcd_con = con
                break

        self.add_file_tags({
            '/etc/etcd/etcd.conf': 'etcd_conf'
        })

        self.add_forbidden_path([
            '/etc/etcd/ca',
            '/etc/etcd/*.key'
        ])

        self.add_dir_listing('/var/lib/etcd/', container=etcd_con,
                             recursive=True)
        self.add_copy_spec('/etc/etcd', container=etcd_con)

        subcmds = [
           'version',
           'member list',
           'alarm list',
           'endpoint status',
           'endpoint health'
        ]

        self.add_cmd_output(
            [f"etcdctl {sub}" for sub in subcmds],
            container=etcd_con
        )

# vim: et ts=5 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class CurtinScrubbedTest(StageTwoReportTest):
    """Ensure that curtin conf is picked up and properly scrubbed

    :avocado: tags=stagetwo
    """

    sos_cmd = '-o curtin'
    files = [
        ('curtin-install-cfg.yaml', '/root/curtin-install-cfg.yaml'),
        ('curtin-install.log', '/root/curtin-install.log'),
    ]

    key_value_obfuscate = {
        "consumer_key": "yaYmBSUuArGt3JYwHU",
        "token_key": "WCCtp2JewdWkHEDAW6",
        "token_secret": "Bm2k3ZdGxFPUK7aZUpTbkRmKQuACKURR",
    }

    files_collect = [
        '/root/curtin-install-cfg.yaml',
        '/root/curtin-install.log',
    ]

    def test_curtin_confs_collected(self):
        for file in self.files_collect:
            self.assertFileCollected(file)

    def test_curtin_files_scrubbed(self):
        for file in self.files_collect:
            for key, value in self.key_value_obfuscate.items():
                self.assertFileNotHasContent(file, fr"{key}: {value}")
                self.assertFileNotHasContent(file, fr"{key}={value}")
                self.assertFileNotHasContent(file, value)
                self.assertFileHasContent(file, fr"{key}:\*\*\*\*\*\*\*\*\*")
                self.assertFileHasContent(file, fr"{key}=\*\*\*\*\*\*\*\*\*")

# vim: set et ts=4 sw=4 :



# Copyright 2024 Red Hat, Inc. Jose Castillo <jcastillo@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import os
import json
from sos.upload.targets import UploadTarget
from sos.utilities import convert_bytes, TIMEOUT_DEFAULT
from sos.policies.auth import DeviceAuthorizationClass
from sos.policies.distros.redhat import RHELPolicy
from sos import _sos as _

try:
    import requests
    REQUESTS_LOADED = True
except ImportError:
    REQUESTS_LOADED = False


class RHELUploadTarget(UploadTarget):

    client_identifier_url = "https://sso.redhat.com/auth/"\
        "realms/redhat-external/protocol/openid-connect/auth/device"
    token_endpoint = "https://sso.redhat.com/auth/realms/"\
        "redhat-external/protocol/openid-connect/token"
    upload_target_name = 'Red Hat Upload Target'
    upload_target_id = "redhat"

    def __init__(self, parser=None, args=None, cmdline=None):

        super().__init__(parser=parser, args=args, cmdline=cmdline)

    RH_API_HOST = "https://api.access.redhat.com"
    RH_SFTP_HOST = "sftp://sftp.access.redhat.com"
    _upload_url = RH_SFTP_HOST
    _upload_method = 'post'
    _device_token = None
    # Max size for an http single request is 1Gb
    _max_size_request = 1073741824

    def check_distribution(self):
        """Return true if we are running in a RHEL system"""
        return isinstance(self.commons['policy'], RHELPolicy)

    def pre_work(self, hook_commons):

        super().pre_work(hook_commons)

        self.upload_directory = self.commons['cmdlineopts'].upload_directory

    def prompt_for_upload_user(self):
        if self.commons['cmdlineopts'].upload_user:
            self.ui_log.info(
                _("The option --upload-user has been deprecated in favour"
                  " of device authorization in RHEL")
            )
        if not self.commons['cmdlineopts'].case_id:
            # no case id provided => failover to SFTP
            self.upload_url = self.RH_SFTP_HOST
            self.ui_log.info("No case id provided, uploading to SFTP")

    def prompt_for_upload_password(self):
        # With OIDC we don't ask for user/pass anymore
        if self.commons['cmdlineopts'].upload_pass:
            self.ui_log.info(
                _("The option --upload-pass has been deprecated in favour"
                  " of device authorization in RHEL")
            )

    def get_upload_url(self):
        rh_case_api = "/support/v1/cases/"\
                     f"{self.commons['cmdlineopts'].case_id}/attachments"
        try:
            if self.upload_url:
                return self.upload_url
            if self.commons['cmdlineopts'].upload_url:
                return self.commons['cmdlineopts'].upload_url
            if self.commons['cmdlineopts'].upload_protocol == 'sftp':
                return self.RH_SFTP_HOST
            if not self.commons['cmdlineopts'].case_id:
                return self.RH_SFTP_HOST

        except Exception as e:
            self.ui_log.info(
                "There was a problem while setting the "
                f"remote upload target:  {e}"
            )
        return f"{self.RH_API_HOST}{rh_case_api}"

    def _get_upload_https_auth(self):
        str_auth = f"Bearer {self._device_token}"
        return {'Authorization': str_auth}

    def _upload_https_post(self, archive, verify=True):
        """If upload_https() needs to use requests.post(), use this method.

        Policies should override this method instead of the base upload_https()

        :param archive:     The open archive file object
        """
        files = {
            'file': (archive.name.split('/')[-1], archive,
                     self._get_upload_headers())
        }
        # Get the access token at this point. With this,
        # we cover the cases where report generation takes
        # longer than the token timeout
        RHELAuth = DeviceAuthorizationClass(
                self.client_identifier_url,
                self.token_endpoint
            )
        self._device_token = RHELAuth.get_access_token()
        self.ui_log.info("Device authorized correctly. Uploading file to "
                         f"{self.get_upload_url_string()}")
        return requests.post(self.get_upload_url(), files=files,
                             headers=self._get_upload_https_auth(),
                             verify=verify, timeout=TIMEOUT_DEFAULT)

    def _get_upload_headers(self):
        if self.get_upload_url().startswith(self.RH_API_HOST):
            return {'isPrivate': 'false', 'cache-control': 'no-cache'}
        return {}

    def get_upload_url_string(self):
        if self.get_upload_url().startswith(self.RH_API_HOST):
            return "Red Hat Customer Portal"
        if self.get_upload_url().startswith(self.RH_SFTP_HOST):
            return "Red Hat Secure FTP"
        return self._get_obfuscated_upload_url(self.upload_url)

    def _get_sftp_upload_name(self):
        """The RH SFTP server will only automatically connect file uploads to
        cases if the filename _starts_ with the case number
        """
        fname = self.upload_archive_name.split('/')[-1]

        if self.commons['cmdlineopts'].case_id:
            fname = f"{self.commons['cmdlineopts'].case_id}_{fname}"
        if self.upload_directory:
            fname = os.path.join(self.upload_directory, fname)
        return fname

    # pylint: disable=too-many-branches
    def upload_sftp(self, user=None, password=None, user_dir=None):
        """Override the base upload_sftp to allow for setting an on-demand
        generated anonymous login for the RH SFTP server if a username and
        password are not given
        """
        if self.RH_SFTP_HOST.split('//')[1] not in self.get_upload_url():
            return super().upload_sftp()

        if not REQUESTS_LOADED:
            raise Exception("python3-requests is not installed and is required"
                            " for obtaining SFTP auth token.")
        _token = None
        _user = None

        # We may have a device token already if we attempted
        # to upload via http but the upload failed. So
        # lets check first if there isn't one.
        if not self._device_token:
            try:
                RHELAuth = DeviceAuthorizationClass(
                    self.client_identifier_url,
                    self.token_endpoint
                )
            except Exception as e:
                # We end up here if the user cancels the device
                # authentication in the web interface
                if "end user denied" in str(e):
                    self.ui_log.info(
                        "Device token authorization "
                        "has been cancelled by the user."
                    )
            else:
                self._device_token = RHELAuth.get_access_token()
        if self._device_token:
            self.ui_log.info("Device authorized correctly. Uploading file to"
                             f" {self.get_upload_url_string()}")

        url = self.RH_API_HOST + '/support/v2/sftp/token'
        ret = None
        if self._device_token:
            headers = self._get_upload_https_auth()
            ret = requests.post(url, headers=headers, timeout=10)
            if ret.status_code == 200:
                # credentials are valid
                _user = json.loads(ret.text)['username']
                _token = json.loads(ret.text)['token']
            else:
                self.ui_log.debug(
                    f"DEBUG: auth attempt failed (status: {ret.status_code}): "
                    f"{ret.json()}"
                )
                self.ui_log.error(
                    "Unable to retrieve Red Hat auth token using provided "
                    "credentials. Will try anonymous."
                )
        else:
            adata = {"isAnonymous": True}
            anon = requests.post(url, data=json.dumps(adata), timeout=10)
            if anon.status_code == 200:
                resp = json.loads(anon.text)
                _user = resp['username']
                _token = resp['token']
                self.ui_log.info(
                    _(f"User {_user} used for anonymous upload. Please inform "
                      f"your support engineer so they may retrieve the data.")
                )
            else:
                self.ui_log.debug(
                    f"DEBUG: anonymous request failed (status: "
                    f"{anon.status_code}): {anon.json()}"
                )
        if _user and _token:
            return super().upload_sftp(user=_user, password=_token,
                                       user_dir=_user)
        raise Exception("Could not retrieve valid or anonymous credentials")

    def check_file_too_big(self, archive):
        size = os.path.getsize(archive)
        # Lets check if the size is bigger than the limit.
        # There's really no need to transform the size to Gb,
        # so we don't need to call any size converter implemented
        # in tools.py
        if size >= self._max_size_request:
            self.ui_log.warning(
                _("Size of archive is bigger than Red Hat Customer Portal "
                  "limit for uploads of "
                  f"{convert_bytes(self._max_size_request)} "
                  " via sos http upload. \n")
                  )
            self.upload_url = self.RH_SFTP_HOST

    def upload_archive(self, archive):
        """Override the base upload_archive to provide for automatic failover
        from RHCP failures to the public RH dropbox
        """
        try:
            if self.get_upload_url().startswith(self.RH_API_HOST):
                self.check_file_too_big(archive)
            uploaded = super().upload_archive(archive)
        except Exception as e:
            uploaded = False
            if not self.upload_url.startswith(self.RH_API_HOST):
                raise
            self.ui_log.error(
                _(f"Upload to Red Hat Customer Portal failed due to "
                  f"{e}. Trying {self.RH_SFTP_HOST}")
                )
            self.upload_url = self.RH_SFTP_HOST
            uploaded = super().upload_archive(archive)
        return uploaded

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
import os
import tempfile
from pathlib import Path


class SoSMap():
    """Standardized way to store items with their obfuscated counterparts.

    Each type of sanitization that SoSCleaner supports should have a
    corresponding SoSMap() object, to allow for easy retrieval of obfuscated
    items.
    """
    # used for regex skips in parser.parse_line()
    ignore_matches = []
    # used for filename obfuscations in parser.parse_string_for_keys()
    skip_keys = []
    compile_regexes = True
    ignore_short_items = False
    match_full_words_only = False

    def __init__(self, workdir):
        self.dataset = {}
        self._regexes_made = set()
        self.compiled_regexes = []
        self.cname = self.__class__.__name__.lower()
        # workdir's default value '/tmp' is used just by avocado tests,
        # otherwise we override it to /etc/sos/cleaner (or map_file dir)
        self.workdir = workdir
        self.cache_dir = os.path.join(self.workdir, 'cleaner_cache',
                                      self.cname)
        self.load_entries()

    def load_entries(self):
        """ Load cached entries from the disk. This method must be called when
        we initialize a Map instance and whenever we want to retrieve
        self.dataset (e.g. to store default_mapping file). The later is
        essential since a concurrent Map can add more objects to the cache,
        so we need to update self.dataset up to date.

        Keep in mind that size of self.dataset is usually bigger than number
        of files in the corresponding cleaner's directory: directory contains
        just whole items (e.g. IP addresses) while dataset contains more
        derived objects (e.g. subnets).
        """

        Path(self.cache_dir).mkdir(parents=True, exist_ok=True)
        self.load_new_entries_from_dir(1)

    def ignore_item(self, item):
        """Some items need to be completely ignored, for example link-local or
        loopback addresses should not be obfuscated
        """
        if not item or item in self.skip_keys or item in self.dataset.values()\
                or (self.ignore_short_items and len(item) <= 3):
            return True
        for skip in self.ignore_matches:
            if re.match(skip, item, re.I):
                return True
        return False

    def add_sanitised_item_to_dataset(self, item):
        try:
            self.dataset[item] = self.sanitize_item(item)
        except Exception:
            self.dataset[item] = item
        if self.compile_regexes:
            self.add_regex_item(item)

    def load_new_entries_from_dir(self, counter):
        # this is a performance hack; there can be gaps in counter values as
        # e.g. sanitised item #14 is an IP address (in file) while item #15
        # is its network (in dataset but not in files). So the next file
        # number is 16. The diffs should be at most 2, the above is so far
        # the only type of "underneath dataset growth". But let be
        # conservative and test next 5 numbers "only".
        no_files_cnt = 5
        while no_files_cnt > 0:
            fname = os.path.join(self.cache_dir, f"{counter}")
            while os.path.isfile(fname):
                no_files_cnt = 5
                with open(fname, 'r', encoding='utf-8') as f:
                    item = f.read()
                if not self.dataset.get(item, False):
                    self.add_sanitised_item_to_dataset(item)
                counter += 1
                fname = os.path.join(self.cache_dir, f"{counter}")
            # no next file, but try a new next ones until no_files_cnt==0
            no_files_cnt -= 1
            counter += 1

    def add(self, item):
        """Add a particular item to the map, generating an obfuscated pair
        for it.

        Positional arguments:

            :param item:        The plaintext object to obfuscate
        """
        if self.ignore_item(item):
            return item

        tmpfile = None
        while not self.dataset.get(item, False):
            if not tmpfile:
                # pylint: disable=consider-using-with
                tmpfile = tempfile.NamedTemporaryFile(dir=self.cache_dir)
                with open(tmpfile.name, 'w', encoding='utf-8') as f:
                    f.write(item)
            try:
                counter = len(self.dataset) + 1
                os.link(tmpfile.name, os.path.join(self.cache_dir,
                                                   f"{counter}"))
                self.add_sanitised_item_to_dataset(item)
            except FileExistsError:
                self.load_new_entries_from_dir(counter)

        return self.dataset[item]

    def add_regex_item(self, item):
        """Add an item to the regexes dict and then re-sort the list that the
        parsers will use during parse_line()

        :param item:    The unobfuscated item to generate a regex for
        :type item:     ``str``
        """
        if self.ignore_item(item):
            return
        if item not in self._regexes_made:
            # save the item in a set to avoid clobbering existing regexes,
            # as searching this set is significantly faster than searching
            # through the actual compiled_regexes list, especially for very
            # large collections of entries
            self._regexes_made.add(item)
            # add the item, Pattern tuple directly to the compiled_regexes list
            # and then sort the existing list, rather than rebuild the list
            # from scratch every time we add something like we would do if we
            # tracked/saved the item and the Pattern() object in a dict or in
            # the set above
            self.compiled_regexes.append((item, self.get_regex_result(item)))
            self.compiled_regexes.sort(key=lambda x: len(x[0]), reverse=True)

    def get_regex_result(self, item):
        """Generate the object/value that is used by the parser when iterating
        over pre-generated regexes during parse_line(). For most parsers this
        will simply be a ``re.Pattern()`` object, but for more complex parsers
        this can be overridden to provide a different object, e.g. a tuple,
        for that parer's specific iteration needs.

        :param item:    The unobfuscated string to generate the regex for
        :type item:     ``str``

        :returns:       A compiled regex pattern for the item
        :rtype:         ``re.Pattern``
        """
        if self.match_full_words_only:
            item = rf'(?<![a-z0-9]){re.escape(item)}(?=\b|_|-)'
        else:
            item = re.escape(item)
        return re.compile(item, re.I)

    def sanitize_item(self, item):
        """Perform the obfuscation relevant to the item being added to the map.

        This should be overridden by each type of map that subclasses SoSMap

        Positional arguments:

            :param item:        The plaintext object to obfuscate
        """
        return item

    def get(self, item):
        """Retrieve an item's obfuscated counterpart from the map. If the item
        does not yet exist in the map, add it by generating one on the fly
        """
        if self.ignore_item(item):
            return item
        if item not in self.dataset:
            return self.add(item)
        return self.dataset[item]

    def conf_update(self, config):
        """Update the map using information from a previous run to ensure that
        we have consistent obfuscation between reports

        Positional arguments:

            :param config:    A dict of mappings with the form of
                              {clean_entry: 'obfuscated_entry'}
        """
        self.dataset.update(config)



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class Lilo(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'Lilo bootloader'

    plugin_name = 'lilo'
    profiles = ('system', 'boot')
    packages = ('lilo',)

    def setup(self):
        self.add_copy_spec("/etc/lilo.conf")
        self.add_cmd_output("lilo -q")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Cron(Plugin, IndependentPlugin):

    short_desc = 'Cron job scheduler'

    plugin_name = "cron"
    profiles = ('system',)
    packages = ('cron', 'anacron', 'chronie')

    files = ('/etc/crontab',)

    def setup(self):
        self.add_copy_spec([
            "/etc/cron*",
            "/var/log/cron",
            "/var/spool/cron"
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/cron*")

        self.add_cmd_output("crontab -l -u root",
                            suggest_filename="root_crontab",
                            tags="root_crontab")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class X11(Plugin, IndependentPlugin):

    short_desc = 'X windowing system'

    plugin_name = 'x11'
    profiles = ('hardware', 'desktop')

    files = ('/etc/X11',)

    def setup(self):
        self.add_copy_spec([
            "/etc/X11",
            "/var/log/Xorg.*.log",
            "/var/log/Xorg.*.log.old",
            "/var/log/XFree86.*.log",
            "/var/log/XFree86.*.log.old",
        ])

        self.add_forbidden_path([
            "/etc/X11/X",
            "/etc/X11/fontpath.d"
        ])

        self.add_cmd_output([
            "xrandr --verbose"
        ])

        self.add_env_var([
            'DISPLAY',
            'DESKTOP_SESSION',
            'XDG_SESSION_TYPE',
            'XDG_SESSION_DESKTOP',
            'XMODIFIERS',
            'XDG_CURRENT_DESKTOP',
            'XDG_SEAT',
            'XDG_RUNTIME_DIR',
            'XAUTHORITY',
            'XDG_SESSION_PATH',
            'XDG_SEAT_PATH',
            'XDG_SESSION_ID'
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import unittest


from sos.report.plugins import import_plugin, PluginOpt
from sos.utilities import ImporterHelper


class PluginConformance(unittest.TestCase):

    def setUp(self):
        # get all plugin classes defined locally
        import sos.report.plugins
        self.plugs = []
        self.plug_classes = []
        helper = ImporterHelper(sos.report.plugins)
        self.plugs = helper.get_modules()
        for plug in self.plugs:
            self.plug_classes.extend(import_plugin(plug))

    def test_plugin_tuples_set_correctly(self):
        for plug in self.plug_classes:
            for tup in ['packages', 'commands', 'files', 'profiles',
                        'kernel_mods', 'containers']:
                _attr = getattr(plug, tup)
                self.assertIsInstance(
                    _attr, tuple,
                    f"{plug.__name__}.{tup} is type {type(_attr)}"
                )

    def test_plugin_description_is_str(self):
        for plug in self.plug_classes:
            self.assertIsInstance(plug.short_desc, str,
                                  f"{plug.__name__} name not string")
            # make sure the description is not empty
            self.assertNotEqual(plug.short_desc, '',
                                f"{plug.__name__} description unset")

    def test_plugin_name_is_str(self):
        for plug in self.plug_classes:
            self.assertIsInstance(plug.plugin_name, str,
                                  f"{plug.__name__} name not string")
            self.assertNotEqual(plug.plugin_name, '',
                                f"{plug.__name__} name unset")

    def test_plugin_option_list_correct(self):
        for plug in self.plug_classes:
            self.assertIsInstance(plug.option_list, list)
            for opt in plug.option_list:
                self.assertIsInstance(opt, PluginOpt)
                self.assertFalse(opt.name == 'undefined')

    def test_plugin_architectures_set_correctly(self):
        for plug in self.plug_classes:
            self.assertIsInstance(plug.architectures, (tuple, type(None)))



# Copyright (C) 2018 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Host(Plugin, IndependentPlugin):
    """This plugin primarily collects hostname related information, as well
    as a few collections that do not fit well in other plugins. For example,
    uptime information and SoS configuration data from /etc/sos.

    This plugin is not intended to be a catch-all "general" plugin however for
    these types of collections that do not have a specific component/package
    or pre-existing plugin.
    """

    short_desc = 'Host information'

    plugin_name = 'host'
    profiles = ('system',)

    def setup(self):

        self.add_forbidden_path('/etc/sos/cleaner')

        self.add_cmd_output('hostname', root_symlink='hostname',
                            tags=['hostname_default', 'hostname_short'])
        self.add_cmd_output('hostname -f', tags='hostname')
        self.add_cmd_output('uptime', root_symlink='uptime', tags="uptime")
        self.add_cmd_output('find / -maxdepth 2 -type l -ls',
                            root_symlink='root-symlinks')

        self.add_cmd_output([
            'hostid',
            'hostnamectl status'
        ])

        self.add_copy_spec([
            '/etc/sos',
            '/etc/hostid',
        ])

        self.add_env_var([
            'REMOTEHOST',
            'TERM',
            'COLORTERM'
        ])



# Copyright (C) 2007 Navid Sheikhol-Eslami <navid@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Radius(Plugin):

    short_desc = 'RADIUS service information'

    plugin_name = "radius"
    profiles = ('network', 'identity')
    packages = ('freeradius',)


class RedHatRadius(Radius, RedHatPlugin):

    files = ('/etc/raddb',)

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/raddb",
            "/etc/pam.d/radiusd",
            "/var/log/radius"
        ])

    def postproc(self):
        self.do_file_sub(
            "/etc/raddb/sql.conf", r"(\s*password\s*=\s*)\S+", r"\1***")


class DebianRadius(Radius, DebianPlugin, UbuntuPlugin):

    files = ('/etc/freeradius',)

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/freeradius",
            "/etc/pam.d/radiusd",
            "/etc/default/freeradius",
            "/var/log/freeradius"
        ])

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import ipaddress

from sos.cleaner.mappings import SoSMap


class SoSIPMap(SoSMap):
    """A mapping store for IP addresses

    Each IP address added to this map is chcked for subnet membership. If that
    subnet already exists in the map, then IP addresses are deterministically
    generated sequentially within that subnet. For example, if a given IP is
    matched to subnet 192.168.1.0/24 then 192.168.1 may be obfuscated to
    100.11.12.0/24. Each IP address in the original 192.168.1.0/24 subnet
    will then be assigned an address in 100.11.12.0/24 sequentially, such as
    100.11.12.1, 100.11.12.2, etc...


    Internally, the ipaddress library is used to manipulate the address objects
    however, when retrieved by SoSCleaner any values will be strings.
    """

    ignore_matches = [
        r'127.*',
        r'::1',
        r'0\.(.*)?',
        r'1\.(.*)?',
        r'8.8.8.8',
        r'8.8.4.4',
        r'169.254.*',
        r'255.*'
    ]

    _networks = {}
    network_first_octet = 100
    skip_network_octets = ['127', '169', '172', '192']
    compile_regexes = False
    # counter for obfuscating a single IP address; the value stands for
    # 172.17.0.0; we use a private block of IP addresses and ignore
    # 172.16.0.0/16 block as those addresses are more often used in real
    # (an attempt to prevent confusion)
    _saddr_cnt = 2886795264

    def ip_in_dataset(self, ipaddr):
        """There are multiple ways in which an ip address could be handed to us
        in a way where we're matching against a previously obfuscated address.

        Here, match the ip address to any of the obfuscated addresses we've
        already created
        """
        for _ip in self.dataset.values():
            if str(ipaddr).split('/', maxsplit=1)[0] == _ip.split('/')[0]:
                return True
        return False

    def get(self, item):
        """Ensure that when requesting an obfuscated address, we return a str
        object instead of an IPv(4|6)Address object
        """
        filt_start = ('/', '=', ']', ')')
        if item.startswith(filt_start):
            item = item.lstrip(''.join(filt_start))

        if item in self.dataset:
            return self.dataset[item]

        if self.ignore_item(item) or self.ip_in_dataset(item):
            return item

        # it's not in there, but let's make sure we haven't previously added
        # an address with a CIDR notation and we're now looking for it without
        # that notation
        if '/' not in item:
            for key, value in self.dataset.items():
                if key.startswith(item):
                    return value.split('/')[0]

        # fallback to the default map behavior of adding it fresh
        return self.add(item)

    def set_ip_cidr_from_existing_subnet(self, addr):
        """Determine if a given address is in a subnet of an already obfuscated
        network and if it is, then set the address' network to the network
        object we're tracking. This allows us to match ip addresses with or
        without a CIDR notation and maintain proper network relationships.
        """
        nets = []
        for net in self._networks:
            if addr.ip == net.broadcast_address:
                addr.network = net
                return
            if addr.ip in net:
                nets.append(net)
        # assign the address to the smallest network that was matched. This is
        # necessary due to certain files specifying addresses that cause the
        # ipaddress library to create artificially huge subnets that will
        # include the actual subnets used by the system
        if nets:
            nets.sort(key=lambda n: n.prefixlen, reverse=True)
            addr.network = nets[0]

    def sanitize_item(self, item):
        """Given an IP address, sanitize it to an obfuscated network or host
        address as appropriate
        """

        try:
            addr = ipaddress.ip_interface(item)
        except ValueError:
            # not an IP, add it to the skip list to avoid flooding logs
            self.ignore_matches.append(item)
            raise
        network = addr.network

        if str(network.netmask) == '255.255.255.255':
            # check to see if this IP is in a subnet of an already obfuscated
            # network and if it has, replace the default /32 netmask that
            # ipaddress applies to no CIDR-notated addresses
            self.set_ip_cidr_from_existing_subnet(addr)
        else:
            # we have a CIDR notation, so generate an obfuscated network
            # address and then generate an IP address within that network's
            # range
            self.sanitize_network(network)
        return self.sanitize_ipaddr(addr)

    def sanitize_network(self, network):
        """Obfuscate the network address provided, and if there are host bits
        in the address then obfuscate those as well
        """
        # check if the address is in a network we've already encountered
        if network not in self._networks:
            self._new_obfuscated_network(network)

    def sanitize_ipaddr(self, addr):
        """Obfuscate the IP address within the known obfuscated network
        """
        # get the obfuscated network object
        if addr.network in self._networks:
            _obf_network = self._networks[addr.network]

            # if the plain address is the broadcast address for it's own
            # network, then assign the broadcast address for the obfuscated
            # network
            if addr.ip == addr.network.broadcast_address:
                return str(_obf_network.broadcast_address)

            # otherwise within that obfuscated network grab the next available
            # address from it
            for _ip in _obf_network.hosts():
                if not self.ip_in_dataset(_ip):
                    # the ipaddress module does not assign the network's
                    # netmask to hosts in the hosts() generator for some reason
                    return f"{str(_ip)}/{_obf_network.prefixlen}"

        # ip is a single ip address without the netmask
        return self._new_obfuscated_single_address()

    def _new_obfuscated_single_address(self):
        # increment the counter and ignore *.0 and *.255 addresses
        self._saddr_cnt += 1
        while self._saddr_cnt % 256 in (0, 255):
            self._saddr_cnt += 1
        # split the counter value to four octets (i.e. % 256) to get an
        # obfuscated IP address
        _addr = f"{self._saddr_cnt >> 24}.{(self._saddr_cnt >> 16) % 256}." \
            f"{(self._saddr_cnt >> 8) % 256}.{self._saddr_cnt % 256}"
        if _addr in self.dataset.values():
            return self._new_obfuscated_single_address()
        return _addr

    def _new_obfuscated_network(self, network):
        """Generate an obfuscated network address for the network address given
        which will allow us to maintain network relationships without divulging
        actual network details

        Positional arguments:

            :param network:     An ipaddress.IPv{4|6)Network object
        """
        _obf_network = None

        if isinstance(network, ipaddress.IPv4Network):
            if self.network_first_octet in self.skip_network_octets:
                self.network_first_octet += 1
            _obf_address = f"{self.network_first_octet}.0.0.0"
            _obf_mask = network.with_netmask.split('/')[1]
            _obf_network = ipaddress.IPv4Network(f"{_obf_address}/{_obf_mask}")
            self.network_first_octet += 1

        if isinstance(network, ipaddress.IPv6Network):
            # TODO: define this
            pass

        if _obf_network:
            self._networks[network] = _obf_network
            self.dataset[str(network)] = str(_obf_network)



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import unittest
import os
import tempfile
import shutil
import random

from io import StringIO
from string import ascii_lowercase
from sos.report.plugins import (Plugin, regex_findall,
                                _mangle_command, PluginOpt)
from sos.archive import TarFileArchive
from sos.policies.distros import LinuxPolicy
from sos.policies.init_systems import InitSystem

PATH = os.path.dirname(__file__)


def j(filename):
    return os.path.join(PATH, filename)


def create_file(size, dirname=None):
    with tempfile.NamedTemporaryFile(delete=False, dir=dirname, mode='w') as f:
        fsize = size * 1024 * 1024
        content = ''.join(random.choice(ascii_lowercase) for x in range(fsize))
        f.write(content)
        f.flush()
        return f.name
    return None


class MockArchive(TarFileArchive):

    # pylint: disable=super-init-not-called
    def __init__(self):
        self.m = {}
        self.strings = {}

    def name(self):
        return "mock.archive"

    def add_file(self, src, dest=None, force=False):
        if not dest:
            dest = src
        self.m[src] = dest

    def add_string(self, content, dest, mode='w'):
        self.m[dest] = content

    def add_link(self, source, link_name):
        pass

    def open_file(self, path):
        return open(self.m.get(path), 'r', encoding='utf-8')

    def close(self):
        pass

    def compress(self, method):
        pass


class MockPlugin(Plugin):

    option_list = [
        PluginOpt("opt", default=None, desc='an option', val_type=str),
        PluginOpt("opt2", default=False, desc='another option')
    ]

    def setup(self):
        pass


class NamedMockPlugin(Plugin):

    short_desc = "This plugin has a description."
    plugin_name = "testing"

    def setup(self):
        pass


class PostprocMockPlugin(Plugin):

    did_postproc = False

    def setup(self):
        pass

    def postproc(self):
        if self.get_option('postproc'):
            self.did_postproc = True


class ForbiddenMockPlugin(Plugin):
    """This plugin has a description."""

    plugin_name = "forbidden"

    def setup(self):
        self.add_copy_spec("tests")
        self.add_forbidden_path("tests")


class EnablerPlugin(Plugin):

    # pylint: disable=unused-argument
    def is_installed(self, package_name):
        return self.is_installed


class MockOptions:
    all_logs = False
    dry_run = False
    since = None
    log_size = 25
    allow_system_changes = False
    no_postproc = False
    skip_files = []
    skip_commands = []
    sysroot = None


class PluginToolTests(unittest.TestCase):

    def test_regex_findall(self):
        test_s = "\n".join(
            ['this is only a test', 'there are only two lines'])
        test_fo = StringIO(test_s)
        matches = regex_findall(r".*lines$", test_fo)
        self.assertEqual(matches, ['there are only two lines'])

    def test_regex_findall_miss(self):
        test_s = "\n".join(
            ['this is only a test', 'there are only two lines'])
        test_fo = StringIO(test_s)
        matches = regex_findall(r".*not_there$", test_fo)
        self.assertEqual(matches, [])

    def test_regex_findall_bad_input(self):
        matches = regex_findall(r".*", None)
        self.assertEqual(matches, [])
        matches = regex_findall(r".*", [])
        self.assertEqual(matches, [])
        matches = regex_findall(r".*", 1)
        self.assertEqual(matches, [])

    def test_mangle_command(self):
        name_max = 255
        self.assertEqual("foo", _mangle_command("/usr/bin/foo", name_max))
        self.assertEqual(
            "foo_-x", _mangle_command("/usr/bin/foo -x", name_max))
        self.assertEqual(
            "foo_--verbose", _mangle_command("/usr/bin/foo --verbose",
                                             name_max))
        self.assertEqual("foo_.path.to.stuff", _mangle_command(
            "/usr/bin/foo /path/to/stuff", name_max))
        longcmd = "foo is " + "a" * 256 + " long_command"
        expected = longcmd[0:name_max].replace(' ', '_')
        self.assertEqual(expected, _mangle_command(longcmd, name_max))


class PluginTests(unittest.TestCase):

    sysroot = os.getcwd()

    def setUp(self):
        self.mp = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.mp.archive = MockArchive()

    def test_plugin_default_name(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.name(), "mockplugin")

    def test_plugin_set_name(self):
        p = NamedMockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.name(), "testing")

    def test_plugin_no_descrip(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_description(), "<no description available>")

    def test_plugin_has_descrip(self):
        p = NamedMockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_description(),
                         "This plugin has a description.")

    def test_set_plugin_option(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        p.set_option("opt", "testing")
        self.assertEqual(p.get_option("opt"), "testing")

    def test_set_nonexistant_plugin_option(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertFalse(p.set_option("badopt", "testing"))

    def test_get_nonexistant_plugin_option(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_option("badopt"), 0)

    def test_get_unset_plugin_option(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_option("opt"), None)

    def test_get_unset_plugin_option_with_default(self):
        # this shows that even when we pass in a default to get,
        # we'll get the option's default as set in the plugin
        # this might not be what we really want
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_option("opt", True), True)

    def test_get_unset_plugin_option_with_default_not_none(self):
        # this shows that even when we pass in a default to get,
        # if the plugin default is not None
        # we'll get the option's default as set in the plugin
        # this might not be what we really want
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        self.assertEqual(p.get_option("opt2", True), False)

    def test_copy_dir(self):
        self.mp._do_copy_path("tests")
        self.assertEqual(
            self.mp.archive.m["tests/unittests/plugin_tests.py"],
            'tests/unittests/plugin_tests.py')

    def test_copy_dir_bad_path(self):
        self.mp._do_copy_path("not_here_tests")
        self.assertEqual(self.mp.archive.m, {})

    def test_copy_dir_forbidden_path(self):
        p = ForbiddenMockPlugin({
            'cmdlineopts': MockOptions(),
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'devices': {}
        })
        p.archive = MockArchive()
        p.setup()
        p.collect_plugin()
        self.assertEqual(p.archive.m, {})

    def test_postproc_default_on(self):
        p = PostprocMockPlugin({
            'cmdlineopts': MockOptions(),
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem()),
            'devices': {}
        })
        p.postproc()
        self.assertTrue(p.did_postproc)

    def test_set_default_cmd_env(self):
        p = MockPlugin({
            'sysroot': self.sysroot,
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })
        e = {'TORVALDS': 'Linus'}
        p.set_default_cmd_environment(e)
        self.assertEqual(p.default_environment, e)
        add_e = {'GREATESTSPORT': 'hockey'}
        p.add_default_cmd_environment(add_e)
        self.assertEqual(p.default_environment['GREATESTSPORT'], 'hockey')
        self.assertEqual(p.default_environment['TORVALDS'], 'Linus')


class AddCopySpecTests(unittest.TestCase):

    expect_paths = set(['tests/unittests/tail_test.txt'])

    def setUp(self):
        self.mp = MockPlugin({
            'cmdlineopts': MockOptions(),
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'sysroot': os.getcwd(),
            'devices': {}
        })
        self.mp.archive = MockArchive()

    def assert_expect_paths(self):
        def pathmunge(path):
            if path[0] == '/':
                path = path[1:]
            return os.path.join(self.mp.sysroot, path)
        expected_paths = set(map(pathmunge, self.expect_paths))
        self.assertEqual(self.mp.copy_paths, expected_paths)

    def test_single_file_no_limit(self):
        self.mp.add_copy_spec("tests/unittests/tail_test.txt")
        self.assert_expect_paths()

    def test_single_file_under_limit(self):
        self.mp.add_copy_spec("tests/unittests/tail_test.txt", 1)
        self.assert_expect_paths()

    def test_single_file_over_limit(self):
        self.mp.sysroot = '/'
        fn = create_file(2)  # create 2MB file, consider a context manager
        self.mp.add_copy_spec(fn, 1)
        fname, _size = self.mp._tail_files_list[0]
        self.assertTrue(fname == fn)
        self.assertTrue("tmp" in fname)
        self.assertEqual(1024 * 1024, _size)
        os.unlink(fn)

    def test_bad_filename(self):
        self.mp.sysroot = '/'
        self.assertFalse(self.mp.add_copy_spec('', 1))
        self.assertFalse(self.mp.add_copy_spec(None, 1))

    def test_glob_file(self):
        self.mp.add_copy_spec('tests/unittests/tail_test.*')
        self.assert_expect_paths()

    def test_glob_file_limit_no_limit(self):
        self.mp.sysroot = '/'
        tmpdir = tempfile.mkdtemp()
        create_file(2, dirname=tmpdir)
        create_file(2, dirname=tmpdir)
        self.mp.add_copy_spec(tmpdir + "/*")
        self.assertEqual(len(self.mp.copy_paths), 2)
        shutil.rmtree(tmpdir)

    def test_glob_file_over_limit(self):
        self.mp.sysroot = '/'
        tmpdir = tempfile.mkdtemp()
        create_file(2, dirname=tmpdir)
        create_file(2, dirname=tmpdir)
        self.mp.add_copy_spec(tmpdir + "/*", 1)
        self.assertEqual(len(self.mp._tail_files_list), 1)
        _, _size = self.mp._tail_files_list[0]
        self.assertEqual(1024 * 1024, _size)
        shutil.rmtree(tmpdir)

    def test_multiple_files_no_limit(self):
        self.mp.add_copy_spec([
            'tests/unittests/tail_test.txt',
            'tests/unittests/test.txt',
        ])
        self.assertEqual(len(self.mp.copy_paths), 2)

    def test_multiple_files_under_limit(self):
        self.mp.add_copy_spec([
            'tests/unittests/tail_test.txt',
            'tests/unittests/test.txt',
        ], 1)
        self.assertEqual(len(self.mp.copy_paths), 2)


class CheckEnabledTests(unittest.TestCase):

    def setUp(self):
        self.mp = EnablerPlugin({
            'policy': LinuxPolicy(probe_runtime=False),
            'sysroot': os.getcwd(),
            'cmdlineopts': MockOptions(),
            'devices': {}
        })

    def test_checks_for_file(self):
        f = j("tail_test.txt")
        self.mp.files = (f,)
        self.assertTrue(self.mp.check_enabled())

    def test_checks_for_package(self):
        self.mp.packages = ('foo',)
        self.assertTrue(self.mp.check_enabled())

    def test_allows_bad_tuple(self):
        f = j("tail_test.txt")
        self.mp.files = f
        self.mp.packages = 'foo'
        self.assertTrue(self.mp.check_enabled())

    def test_enabled_by_default(self):
        self.assertTrue(self.mp.check_enabled())


class RegexSubTests(unittest.TestCase):

    def setUp(self):
        self.mp = MockPlugin({
            'cmdlineopts': MockOptions(),
            'policy': LinuxPolicy(init=InitSystem(), probe_runtime=False),
            'sysroot': os.getcwd(),
            'devices': {}
        })
        self.mp.archive = MockArchive()

    def test_file_never_copied(self):
        self.assertEqual(0, self.mp.do_file_sub(
            "never_copied", r"^(.*)$", "foobar"))

    def test_no_replacements(self):
        self.mp.sysroot = '/'
        self.mp.add_copy_spec(j("tail_test.txt"))
        self.mp.collect_plugin()
        replacements = self.mp.do_file_sub(
            j("tail_test.txt"), r"wont_match", "foobar")
        self.assertEqual(0, replacements)

    def test_replacements(self):
        # test uses absolute paths
        self.mp.sysroot = '/'
        self.mp.add_copy_spec(j("tail_test.txt"))
        self.mp.collect_plugin()
        replacements = self.mp.do_file_sub(
            j("tail_test.txt"), r"(tail)", "foobar")
        self.assertEqual(1, replacements)
        self.assertTrue("foobar" in self.mp.archive.m.get(j('tail_test.txt')))


if __name__ == "__main__":
    unittest.main()

# vim: set et ts=4 sw=4 :



# Copyright (C) 2025 Red Hat, Inc., David Wolstromer <dwolstro@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Valkey(Plugin, IndependentPlugin):

    short_desc = 'Valkey, in-memory data structure store'

    plugin_name = 'valkey'
    profiles = ('services',)

    packages = ('valkey',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/valkey"

    def setup(self):
        self.add_copy_spec([
            "/etc/valkey/*",
            self.var_puppet_gen + "/etc/valkey*",
            self.var_puppet_gen + "/etc/valkey/",
            self.var_puppet_gen + "/etc/security/limits.d/"
        ])

        self.add_cmd_output("valkey-cli info")
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/valkey/valkey.log*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/valkey/valkey.log",
            ])

    def postproc(self):
        for path in ["/etc/valkey/", self.var_puppet_gen + "/etc/valkey"]:
            self.do_file_sub(
                path + "valkey.conf",
                r"(primaryauth|requirepass|key-file-pass)\s.*",
                r"\1 ********"
            )
            self.do_file_sub(
                path + "sentinel.conf",
                r"(primaryauth|requirepass|key-file-pass)\s.*",
                r"\1 ********"
            )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin, RedHatPlugin
from sos.utilities import is_executable


class Slurm(Plugin, UbuntuPlugin, RedHatPlugin):

    short_desc = "Slurm Workload Manager"

    plugin_name = 'slurm'
    profiles = ('hpc',)
    packages = (
        # Ubuntu
        'slurm-wlm',
        'slurmd',
        'slurmdbd',
        'slurmctld',
        # EL
        'slurm',
        'slurm-slurmctld',
        'slurm-slurmd',
        'slurm-slurmdbd',
    )
    services = (
        'slurmd',
        'slurmdbd',
        'slurmctld',
    )

    def setup(self):
        """ Slurm Workload Manager
        """

        self.add_copy_spec([
            '/etc/slurm/*.conf',
            '/var/run/slurm/conf/*.conf',
        ])

        if is_executable('sinfo'):
            self.add_cmd_output([
                'sinfo --all --list-reasons --long',
                'sinfo --all --long',
            ])

        if is_executable('squeue'):
            self.add_cmd_output([
                'squeue --all --long',
            ])

        scontrol_cmds = [
            'aliases',
            'assoc_mgr',
            'bbstat',
            'burstBuffer',
            'config',
            'daemons',
            'dwstat',
            'federation',
            'frontend',
            'job',
            'licenses',
            'node',
            'partition',
            'reservation',
            'slurmd',
            'step',
            'topology',
        ]

        if is_executable('scontrol'):
            self.add_cmd_output(
                [f"scontrol show {i}" for i in scontrol_cmds]
            )

        config_file = '/etc/slurm/slurm.conf'

        if not self.path_exists(config_file):
            config_file = '/var/run/slurm/conf/slurm.conf'

        slurmd_log_file = '/var/log/slurmd.log'
        slurmctld_log_file = '/var/log/slurmctld.log'

        try:
            with open(config_file, 'r', encoding='UTF-8') as cfile:
                for line in cfile.read().splitlines():
                    if not line:
                        continue
                    words = line.split('=')
                    if words[0].strip() == 'SlurmdLogFile':
                        slurmd_log_file = words[1].strip()
                    if words[0].strip() == 'SlurmctldLogFile':
                        slurmctld_log_file = words[1].strip()
        except IOError as error:
            self._log_error(f'Could not open conf file {config_file}:'
                            f' {error}')

        if not self.get_option("all_logs"):
            self.add_copy_spec([
                slurmd_log_file,
                slurmctld_log_file,
            ])
        else:
            self.add_copy_spec([
                f"{slurmd_log_file}*",
                f"{slurmctld_log_file}*",
            ])

    def postproc(self):
        conf_paths = [
            "/etc/slurm",
            "/var/run/slurm/conf",
        ]

        slurm_keys = [
            'AccountingStoragePass',
            'JobCompPass',
        ]
        slurm_keys_regex = fr"(^\s*({'|'.join(slurm_keys)})\s*=\s*)(.*)"
        slurmdbd_key_regex = r'(^\s*(StoragePass)\s*=\s*)(.*)'

        sub = r'\1********'

        for conf_path in conf_paths:
            self.do_file_sub(
                f'{conf_path}/slurm.conf',
                slurm_keys_regex, sub
            )
            self.do_file_sub(
                f'{conf_path}/slurmdbd.conf',
                slurmdbd_key_regex, sub
            )



# Copyright (C) 2018 Mikel Olasagasti Uranga <mikel@olasagasti.info>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Memcached(Plugin):

    short_desc = 'memcached distributed memory caching system'

    plugin_name = 'memcached'
    profiles = ('webserver',)
    packages = ('memcached',)

    mm_tool = "memcached-tool"

    def setup(self):
        host = "127.0.0.1:11211"

        subcmds = [
            "display",
            "dump",
            "keys",
            "stats",
            "settings",
        ]

        self.add_cmd_output([
            f"{self.mm_tool} {host} {subcmd}" for subcmd in subcmds])


class RedHatMemcached(Memcached, RedHatPlugin):

    files = ('/etc/sysconfig/memcached',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/sysconfig/memcached",
                           tags="sysconfig_memcached")


class DebianMemcached(Memcached, DebianPlugin, UbuntuPlugin):

    files = ('/etc/default/memcached',)

    def setup(self):
        self.mm_tool = "/usr/share/memcached/scripts/memcached-tool"
        super().setup()
        self.add_copy_spec([
            "/etc/memcached.conf",
            "/etc/default/memcached"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import pwd

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Cups(Plugin, IndependentPlugin):

    short_desc = 'CUPS IPP print service'

    plugin_name = 'cups'
    profiles = ('hardware',)
    services = ('cups', 'cups-browsed',
                'lprint', 'legacy-printer-app')
    packages = ('cups',)

    option_list = [
        PluginOpt('userconfs', default=False,
                  desc=('Changes whether plugin will '
                        'collect user .cups configs'))
    ]

    def setup(self):
        if not self.get_option("all_logs"):
            self.add_copy_spec("/var/log/cups/access_log")
            self.add_copy_spec("/var/log/cups/error_log")
            self.add_copy_spec("/var/log/cups/page_log")
            self.add_copy_spec("/var/log/ipp-usb/main.log")
        else:
            self.add_copy_spec("/var/log/cups")
            self.add_copy_spec("/var/log/ipp-usb")

        self.add_copy_spec([
            "/etc/cups/*.conf",
            "/etc/cups/*.types",
            "/etc/cups/lpoptions",
            "/etc/cups/ppd/*.ppd",
            "/etc/ipp-usb/",
            "/etc/lprint.conf",
            "/etc/legacy-printer-app.conf",
            "/var/lib/lprint.state",
            "/var/lib/legacy-printer-app.state",
        ])

        self.add_cmd_output([
            "lpstat -t",
            "lpstat -s",
            "lpstat -d"
        ])

        if self.get_option('userconfs'):
            self.get_user_configs()

    def get_user_configs(self):
        """
        Iterate over .cups folders in user homes to capture config files.
        """
        users_data = pwd.getpwall()
        config_files = [
            "client.conf",
            "lpoptions",
        ]
        fs_mount_info = {}
        try:
            with open('/proc/mounts', "r", encoding='UTF-8') as mounts_file:
                for line in mounts_file:
                    (fs_file, fs_vstype) = line.split()[1:3]
                    fs_mount_info[fs_file] = fs_vstype
        except Exception:
            self._log_error("Couldn't read /proc/mounts")
            return
        non_local_fs = {'nfs', 'nfs4', 'autofs'}
        # Read the home paths of users in the system and
        # config files from .cups
        for user in users_data:
            if user.pw_dir in fs_mount_info and \
                    fs_mount_info[user.pw_dir] in non_local_fs:
                self._log_info(
                    f"Skipping capture in {user.pw_dir}"
                    " because it's a remote directory"
                )
                continue
            home_dir = self.path_join(user.pw_dir, '.cups')
            self.add_copy_spec(
                [f"{home_dir}/{config_file}" for config_file in config_files]
            )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class DefaultCollectionsTest(StageTwoReportTest):
    """Ensure that the default collections are firing for Plugins based on
    their enablement triggers, which gives us more atomicity in Plugin design

    :avocado: tags=stagetwo
    """

    packages = {
        'rhel': 'cups',
        'Ubuntu': 'cups',
        'debian': 'cups',
    }

    sos_cmd = '-o cups'

    def test_service_status_collected(self):
        self.assertFileCollected('sos_commands/cups/systemctl_status_cups')
        _m = self.get_plugin_manifest('cups')
        ent = None
        for cmd in _m['commands']:
            if cmd['exec'] == 'systemctl status cups':
                ent = cmd
        assert ent, "No manifest entry for systemctl status cups"

    def test_journal_collected(self):
        self.assertFileCollected('sos_commands/cups/journalctl_--no-pager_'
                                 '--unit_cups')
        _m = self.get_plugin_manifest('cups')
        ent = None
        for cmd in _m['commands']:
            if cmd['exec'] == 'journalctl --no-pager  --unit cups --reverse':
                ent = cmd
        assert ent, "No manifest entry for journalctl cups"

        assert \
            'journal_cups' in ent['tags'], \
            f"Journal tags not correct: {ent['tags']}"



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from sos.cleaner.mappings import SoSMap


class SoSMacMap(SoSMap):
    """Mapping store for MAC addresses

    MAC addresses added to this map will be broken into two halves, vendor and
    device like how MAC addresses are normally crafted. For the vendor hextets,
    obfuscation will take the form of 53:4f:53, or 'SOS' in hex. The following
    device hextets will be obfuscated by a series of suffixes starting from
    zeroes. For example a MAC address of '60:55:cb:4b:c9:27' may be obfuscated
    into '53:4f:53:00:00:1a' or similar.

    This map supports both 48-bit and 64-bit MAC addresses.

    48-bit address may take the form of either:

            MM:MM:MM:SS:SS:SS
            MM-MM-MM-SS-SS-SS

    For 64-bit addresses, the identifier injected by IPv6 standards is
    used in obfuscated returns. These addresses may take either of these forms:

        MM:MM:MM:FF:FE:SS:SS:SS
        MMMM:MMFF:FESS:SSSS

    All mapped mac addresses are converted to lower case.
    Dash delimited styles will be converted to colon-delimited style.
    """

    ignore_matches = [
        'ff:ff:ff:ff:ff:ff',
        '00:00:00:00:00:00'
    ]

    mac_template = '53:4f:53:%s:%s:%s'
    mac6_template = '53:4f:53:ff:fe:%s:%s:%s'
    mac6_quad_template = '534f:53ff:fe%s:%s%s'
    compile_regexes = False
    ob_hextets_cnt = 0

    def add(self, item):
        item = item.replace('-', ':').lower().strip('=.,').strip()
        return super().add(item)

    def get(self, item):
        item = item.replace('-', ':').lower().strip('=.,').strip()
        return super().get(item)

    def sanitize_item(self, item):
        """Obfuscate the device hextets, and append those to our 'vendor'
        hextet
        """
        hexdigits = "0123456789abdcef"
        self.ob_hextets_cnt += 1
        # we need to convert the counter to a triple of double hex-digits
        hextets = [
            self.ob_hextets_cnt >> 16,
            (self.ob_hextets_cnt >> 8) % 256,
            self.ob_hextets_cnt % 256
        ]
        hextets = tuple(f'{hexdigits[i//16]}{hexdigits[i % 16]}'
                        for i in hextets)

        # match 64-bit IPv6 MAC addresses matching MM:MM:MM:FF:FE:SS:SS:SS
        if re.match('(([0-9a-fA-F]{2}:){7}[0-9a-fA-F]{2})', item):
            return self.mac6_template % hextets
        # match 64-bit IPv6 MAC addresses matching MMMM:MMFF:FESS:SSSS
        if re.match('(([0-9a-fA-F]{4}:){3}([0-9a-fA-F]){4})', item):
            return self.mac6_quad_template % hextets
        # match 48-bit IPv4 MAC addresses
        if re.match('([0-9a-fA-F][:_]?){12}', item):
            return self.mac_template % hextets
        return None



# Copyright 2020 Red Hat, Inc.
# Author: Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import logging
import os
import tempfile
import sys
import time

from textwrap import fill
from argparse import SUPPRESS
from datetime import datetime
from getpass import getpass
from shutil import rmtree
from pathlib import Path
from sos import __version__
from sos.archive import TarFileArchive
from sos.options import SoSOptions
from sos.utilities import TempFileUtil, shell_out


class SoSComponent():
    """Any sub-command that sos supports needs to subclass SoSComponent in
    order to be properly supported by the sos binary.

    This class contains the standardized entrypoint for subcommands, as well as
    building out supported options from both globally shared option lists, and
    options supported by that specific subcommand.

    When sos initializes, it will load an unintialized instance of each class
    found within one recursion of the module root directory that subclasses
    SoSComponent.

    If sos is able to match the user-specified subcommand to one that exists
    locally, then that SoSComponent is initialized, logging is setup, and a
    policy is loaded. From there, the component's execute() method takes over.

    Added in 4.0
    """

    desc = 'unset'

    arg_defaults = {}
    configure_logging = True
    load_policy = True
    load_probe = True
    root_required = False

    _arg_defaults = {
        "batch": False,
        "compression_type": 'auto',
        "config_file": '/etc/sos/sos.conf',
        "debug": False,
        "encrypt": False,
        "encrypt_key": None,
        "encrypt_pass": None,
        "quiet": False,
        "threads": 4,
        "tmp_dir": '',
        "sysroot": None,
        "verbosity": 0
    }

    # files in collected archive that might contain upload password
    files_with_upload_passwd = [
        "sos_logs/sos.log",
        "sos_reports/manifest.json",
        "sos_commands/process/ps_*",
        "sos_commands/selinux/ps_*",
        "sos_commands/systemd/systemctl_status_--all",
    ]

    def __init__(self, parser, parsed_args, cmdline_args):
        self.parser = parser
        self.args = parsed_args
        self.cmdline = cmdline_args
        self.exit_process = False
        self.archive = None
        self.tmpdir = None
        self.tempfile_util = None
        self.manifest = None

        try:
            import signal
            signal.signal(signal.SIGTERM, self.get_exit_handler())
        except Exception as err:
            sys.stdout.write(f"Notice: Could not set SIGTERM handler: {err}\n")

        self.opts = SoSOptions(arg_defaults=self._arg_defaults)
        if self.load_policy:
            self.load_local_policy()

        # update args from component's arg_defaults definition
        self._arg_defaults.update(self.arg_defaults)
        self.opts = self.load_options()  # lgtm [py/init-calls-subclass]

        if self.configure_logging:
            tmpdir = self.get_tmpdir_default()
            # only setup metadata if we are logging
            self.manifest = SoSMetadata()

            if not os.path.isdir(tmpdir) \
                    or not os.access(tmpdir, os.W_OK):
                msg = f"temporary directory {tmpdir} "
                msg += "does not exist or is not writable\n"
                # write directly to stderr as logging is not initialised yet
                sys.stderr.write(msg)
                self._exit(1)

            self.sys_tmp = tmpdir

            self.tmpdir = tempfile.mkdtemp(prefix="sos.", dir=self.sys_tmp)
            self.tempfile_util = TempFileUtil(self.tmpdir)
            self._setup_logging()

        if self.manifest is not None:
            self.manifest.add_field('version', __version__)
            self.manifest.add_field('cmdline', ' '.join(self.cmdline))
            self.manifest.add_field('start_time', datetime.now())
            # these three will be set later, add here for organization
            self.manifest.add_field('end_time', '')
            self.manifest.add_field('run_time', '')
            self.manifest.add_field('compression', '')
            self.manifest.add_field('tmpdir', self.tmpdir)
            self.manifest.add_field('tmpdir_fs_type', self.tmpfstype)
            self.manifest.add_field('policy', self.policy.os_release_name)
            self.manifest.add_section('components')

    def load_local_policy(self):
        try:
            import sos.policies
            self.policy = sos.policies.load(sysroot=self.opts.sysroot,
                                            probe_runtime=self.load_probe)
            self.sysroot = self.policy.sysroot
        except KeyboardInterrupt:
            self._exit(0)
        self._is_root = self.policy.is_root()

    def execute(self):
        raise NotImplementedError

    def get_exit_handler(self):
        def exit_handler(signum, frame):  # pylint: disable=unused-argument
            self.exit_process = True
            self._exit()
        return exit_handler

    def _exit(self, error=0, msg=None):
        if msg:
            self.ui_log.error("")
            self.ui_log.error(msg)
        raise SystemExit(error)

    def get_tmpdir_default(self):
        """If --tmp-dir is not specified, provide a default location.
        Normally this is /var/tmp, but if we detect we are in a container, then
        use a standardized env var to redirect to the host's filesystem instead
        """
        if self.opts.tmp_dir:
            tmpdir = os.path.abspath(self.opts.tmp_dir)
        else:
            tmpdir = os.getenv('TMPDIR', None) or self.policy.get_tmp_dir(None)

        if os.getenv('HOST', None) and os.getenv('container', None):
            tmpdir = os.path.join(os.getenv('HOST'), tmpdir.lstrip('/'))

        # no standard library method exists for this, so call out to stat to
        # avoid bringing in a dependency on psutil
        self.tmpfstype = shell_out(
            f"stat --file-system --format=%T {tmpdir}"
        ).strip()

        if self.tmpfstype == 'tmpfs':
            # can't log to the ui or sos.log yet since those require a defined
            # tmpdir to setup
            print("WARNING: tmp-dir is set to a tmpfs filesystem. This may "
                  "increase memory pressure and cause instability on low "
                  "memory systems, or when using --all-logs.")
            time.sleep(2)

        return tmpdir

    def check_listing_options(self):
        opts = [o for o in self.opts.dict().keys() if o.startswith('list')]
        if opts:
            return any(getattr(self.opts, opt) for opt in opts)
        return False

    @classmethod
    def add_parser_options(cls, parser):
        """This should be overridden by each subcommand to add its own unique
        options to the parser
        """
        raise NotImplementedError

    def apply_options_from_cmdline(self, opts):
        """(Re-)apply options specified via the cmdline to an options instance

        There are several cases where we may need to re-apply the options from
        the cmdline over previously loaded options - for instance when an
        option is specified in both a config file and cmdline, or a preset and
        the cmdline, or all three.

        Use this to re-apply cmdline option overrides to anything that may
        change the default values of options

        Positional arguments:

            :param opts:        SoSOptions object to update

        """
        # override the values from config files with the values from the
        # cmdline iff that value was explicitly specified, and compare it to
        # the _current_ set of opts from the config files as a default
        cmdopts = SoSOptions().from_args(
            self.parser.parse_args(self.cmdline),
            arg_defaults=opts.dict(preset_filter=False)
        )
        # we can't use merge() here, as argparse will pass default values for
        # unset options which would effectively negate config file settings and
        # set all values back to their normal default
        codict = cmdopts.dict(preset_filter=False)
        for opt, val in codict.items():
            if opt not in cmdopts.arg_defaults.keys() or val in [None, [], '']:
                continue
            # A plugin that is [enabled|disabled|only] in cmdopts must
            # overwrite these three options of itself in opts - reset it first
            if opt in ["enable_plugins", "skip_plugins", "only_plugins"]:
                for oopt in ["enable_plugins", "skip_plugins", "only_plugins"]:
                    common = set(val) & set(getattr(opts, oopt))
                    # common has all plugins that are in this combination of
                    # "[-e|-o|-n] plug" of cmdopts & "[-e|-o|-n] plug" of opts
                    # so remove those plugins from this [-e|-o|-n] opts
                    if common:
                        setattr(opts, oopt, [x for x in getattr(opts, oopt)
                                if x not in common])

            # plugin options as a list should be concatenated, not overriden
            # BUT if cmdline plugoption overrides same option in opts, we must
            # drop the opts's value; since the items are in form
            # 'apache.log=on', we must separate the *name* of each option
            if opt == 'plugopts':
                oplugopts = getattr(opts, opt)
                valnames = [v.split('=')[0] for v in val]
                ovalnames = [v.split('=')[0] for v in oplugopts]
                for common in set(valnames) & set(ovalnames):
                    cstring = f"{common}="
                    oplugopts = [oopt for oopt in oplugopts
                                 if not oopt.startswith(cstring)]
                setattr(opts, opt, list(set(val) | set(oplugopts)))
            elif val != opts.arg_defaults[opt]:
                setattr(opts, opt, val)

        return opts

    def load_options(self):
        """Compile arguments loaded from defaults, config files, and the
        command line into a usable set of options
        """
        # load the defaults defined by the component and the shared options
        opts = SoSOptions(arg_defaults=self._arg_defaults)

        for option in self.parser._actions:
            if option.default != SUPPRESS:
                option.default = None

        opts.update_from_conf(self.args.config_file, self.args.component)

        # directly check the cmdline options here as they have yet to be loaded
        # as SoSOptions, and if we do this check after they are loaded we would
        # need to do a second update from cmdline options for overriding config
        # file values
        if '--clean' in self.cmdline or '--mask' in self.cmdline:
            opts.update_from_conf(self.args.config_file, 'clean')

        if os.getuid() != 0:
            userconf = os.path.join(Path.home(), '.config/sos/sos.conf')
            if os.path.exists(userconf):
                opts.update_from_conf(userconf, self.args.component)

        opts = self.apply_options_from_cmdline(opts)

        # user specified command line preset
        self.preset = None
        if hasattr(opts, 'preset'):
            if opts.preset != self._arg_defaults["preset"]:
                self.preset = self.policy.find_preset(opts.preset)
                if not self.preset:
                    sys.stderr.write(f"Unknown preset: '{opts.preset}'\n")
                    self.preset = self.policy.probe_preset()
                    opts.list_presets = True

            # --preset=auto
            if not self.preset:
                self.preset = self.policy.probe_preset()
            # now merge preset options to opts
            opts.merge(self.preset.opts)
            # re-apply any cmdline overrides to the preset
            opts = self.apply_options_from_cmdline(opts)

            if hasattr(self.preset.opts, 'verbosity') and \
                    self.preset.opts.verbosity > 0:
                self.set_loggers_verbosity(self.preset.opts.verbosity)

        return opts

    def cleanup(self):
        # archive and tempfile cleanup may fail due to a fatal
        # OSError exception (ENOSPC, EROFS etc.).
        try:
            if self.archive:
                self.archive.cleanup()
            if self.tempfile_util:
                self.tempfile_util.clean()
            if self.tmpdir:
                rmtree(self.tmpdir)
        except Exception as err:
            print(f"Failed to finish cleanup: {err}\nContents may remain in "
                  f"{self.tmpdir}")

    def _set_encrypt_from_env_vars(self):
        msg = ('No encryption environment variables set, archive will not be '
               'encrypted')
        if os.environ.get('SOSENCRYPTKEY'):
            self.opts.encrypt_key = os.environ.get('SOSENCRYPTKEY')
            msg = 'Encryption key set via environment variable'
        elif os.environ.get('SOSENCRYPTPASS'):
            self.opts.encrypt_pass = os.environ.get('SOSENCRYPTPASS')
            msg = 'Encryption passphrase set via environment variable'
        self.soslog.info(msg)
        self.ui_log.info(msg)

    def _get_encryption_method(self):
        if not self.opts.batch:
            _enc = None
            while _enc not in ('P', 'K', 'E', 'N'):
                _enc = input((
                    'Specify encryption method [P]assphrase, [K]ey, [E]nv '
                    'vars, [N]o encryption: '
                )).upper()
            if _enc == 'P':
                self.opts.encrypt_pass = getpass('Specify encryption '
                                                 'passphrase: ')
            elif _enc == 'K':
                self.opts.encrypt_key = input('Specify encryption key: ')
            elif _enc == 'E':
                self._set_encrypt_from_env_vars()
            else:
                self.opts.encrypt_key = None
                self.opts.encrypt_pass = None
                self.soslog.info("User specified --encrypt, but chose no "
                                 "encryption when prompted.")
                self.ui_log.warning("Archive will not be encrypted")
        else:
            self._set_encrypt_from_env_vars()

    def setup_archive(self, name=''):
        if self.opts.encrypt:
            self._get_encryption_method()
        enc_opts = {
            'encrypt': self.opts.encrypt_pass or self.opts.encrypt_key,
            'key': self.opts.encrypt_key,
            'password': self.opts.encrypt_pass
        }
        if not name:
            name = self.policy.get_archive_name()
        archive_name = os.path.join(self.tmpdir, name)
        if self.opts.compression_type == 'auto':
            auto_archive = self.policy.get_preferred_archive()
            self.archive = auto_archive(archive_name, self.tmpdir,
                                        self.policy, self.opts.threads,
                                        enc_opts, self.sysroot,
                                        self.manifest)

        else:
            self.archive = TarFileArchive(archive_name, self.tmpdir,
                                          self.policy, self.opts.threads,
                                          enc_opts, self.sysroot,
                                          self.manifest)

        self.archive.set_debug(self.opts.verbosity > 2)

    def _obfuscate_upload_passwords(self):
        # obfuscate strings like:
        # --upload-pass=PASSWORD
        # --upload-pass PASSWORD
        # --upload-url https://user:PASSWORD@some.url
        # in both sos_logs/sos.log and in sos_reports/manifest.json
        # and several sos_commands/* places from plugins's collected data
        _arc_path = self.archive.get_archive_path()
        for path in self.files_with_upload_passwd:
            for f in Path(_arc_path).glob(path):
                # get just the relative path that archive works with
                f = os.path.relpath(f, _arc_path)
                for re in [r"(--upload-pass[\s=]+)\S+",
                           r"(--upload-url[\s=]+\S+://.*:)([^@]*)",
                           r"(--upload-s3-secret-key[\s=]+)\S+"]:
                    self.archive.do_file_sub(f, re, r"\1********")

    def add_ui_log_to_stdout(self):
        ui_console = logging.StreamHandler(sys.stdout)
        ui_console.setFormatter(logging.Formatter('%(message)s'))
        ui_console.setLevel(
            logging.DEBUG if self.opts.verbosity > 1 else logging.INFO
        )
        self.ui_log.addHandler(ui_console)

    def set_loggers_verbosity(self, verbosity):
        if getattr(self, 'flog', None) and verbosity:
            self.flog.setLevel(logging.DEBUG)
        if getattr(self, 'console', None):
            if verbosity and self.opts.verbosity > 1:
                self.console.setLevel(logging.DEBUG)
            else:
                self.console.setLevel(logging.WARNING)

    def _setup_logging(self):
        """Creates the log handler that shall be used by all components and any
        and all related bits to those components that need to log either to the
        console or to the log file for that run of sos.
        """
        # main soslog
        self.soslog = logging.getLogger('sos')
        self.soslog.setLevel(logging.DEBUG)
        self.flog = None
        if not self.check_listing_options():
            self.sos_log_file = self.get_temp_file()
            self.flog = logging.StreamHandler(self.sos_log_file)
            self.flog.setFormatter(logging.Formatter(
                '%(asctime)s %(levelname)s: %(message)s'))
            self.flog.setLevel(logging.INFO)
            self.soslog.addHandler(self.flog)

        if not self.opts.quiet:
            self.console = logging.StreamHandler(sys.stdout)
            self.console.setFormatter(logging.Formatter('%(message)s'))
            self.set_loggers_verbosity(self.opts.verbosity)
            self.soslog.addHandler(self.console)
        # still log ERROR level message to console, but only setup this handler
        # when --quiet is used, as otherwise we'll double log
        else:
            console_err = logging.StreamHandler(sys.stderr)
            console_err.setFormatter(logging.Formatter('%(message)s'))
            console_err.setLevel(logging.ERROR)
            self.soslog.addHandler(console_err)

        # ui log
        self.ui_log = logging.getLogger('sos_ui')
        self.ui_log.setLevel(
            logging.DEBUG if self.opts.verbosity > 1 else logging.INFO
        )
        if not self.check_listing_options():
            self.sos_ui_log_file = self.get_temp_file()
            ui_fhandler = logging.StreamHandler(self.sos_ui_log_file)
            ui_fhandler.setFormatter(logging.Formatter(
                '%(asctime)s %(levelname)s: %(message)s'))

            self.ui_log.addHandler(ui_fhandler)

        if not self.opts.quiet:
            self.add_ui_log_to_stdout()

    def get_temp_file(self):
        return self.tempfile_util.new()

    def _fmt_msg(self, msg):
        width = 80
        _fmt = ''
        for line in msg.splitlines():
            _fmt = _fmt + fill(line, width, replace_whitespace=False) + '\n'
        return _fmt


class SoSMetadata():
    """This class is used to record metadata from a sos execution that will
    then be stored as a JSON-formatted manifest within the final tarball.

    It can be extended by adding further instances of SoSMetadata to represent
    dict-like structures throughout the various sos bits that record to
    metadata
    """

    def __init__(self):
        self._values = {}

    def __iter__(self):
        for item in self._values.items():
            yield item[1]

    def __getitem__(self, item):
        return self._values[item]

    def __getattr__(self, attr):
        return self._values[attr]

    def add_field(self, field_name, content):
        """Add a key, value entry to the current metadata instance
        """
        self._values[field_name] = content

    def add_section(self, section_name):
        """Adds a new instance of SoSMetadata to the current instance
        """
        self._values[section_name] = SoSMetadata()
        return self._values[section_name]

    def add_list(self, list_name, content=[]):
        """Add a named list element to the current instance. If content is not
        supplied, then add an empty list that can alter be appended to
        """
        if not isinstance(content, list):
            raise TypeError('content added must be list')
        self._values[list_name] = content

    def get_json(self, indent=None):
        """Convert contents of this SoSMetdata instance, and all other nested
        instances (sections), into a json-formatted output.

        Used to write manifest.json to the final archives.
        """
        return json.dumps(self,
                          default=lambda o: getattr(o, '_values', str(o)),
                          indent=indent)

# vim: set et ts=4 sw=4 :



# Copyright Red Hat 2020, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from shlex import quote
from sos.collector.clusters import Cluster


class kubernetes(Cluster):
    """
    The kuberentes cluster profile is intended to be used on kubernetes
    clusters built from the upstream/source kubernetes (k8s) project. It is
    not intended for use with other projects or platforms that are built ontop
    of kubernetes.
    """
    cluster_name = 'Community Kubernetes'
    packages = ('kubernetes-master',)
    sos_plugins = ['kubernetes']
    sos_plugin_options = {'kubernetes.all': 'on'}

    cmd = 'kubectl'

    option_list = [
        ('label', '', 'Filter node list to those with matching label'),
        ('role', '', 'Filter node list to those with matching role')
    ]

    def get_nodes(self):
        self.cmd += ' get nodes'
        if self.get_option('label'):
            self.cmd += f' -l {quote(self.get_option("label"))} '
        res = self.exec_primary_cmd(self.cmd)
        if res['status'] == 0:
            nodes = []
            roles = [x for x in self.get_option('role').split(',') if x]
            for nodeln in res['output'].splitlines()[1:]:
                node = nodeln.split()
                if not roles:
                    nodes.append(node[0])
                else:
                    if node[2] in roles:
                        nodes.append(node[0])
            return nodes
        raise Exception('Node enumeration did not return usable output')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from avocado.core.exceptions import TestSkipError
from avocado.utils import process
from sos_tests import StageOneReportTest, redhat_only


class NormalSoSReport(StageOneReportTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '--label thisismylabel'

    def test_debug_in_logs_verbose(self):
        self.assertSosLogContains('DEBUG')

    def test_debug_not_printed_to_console(self):
        self.assertOutputNotContains('added cmd output')
        self.assertOutputNotContains(r'\[archive:.*\]')

    def test_postproc_called(self):
        self.assertSosLogContains('substituting scrpath')

    def test_label_applied_to_archive(self):
        self.assertTrue('thisismylabel' in self.archive)

    def test_free_symlink_created(self):
        self.assertFileCollected('free')

    def test_tag_summary_created(self):
        self.assertTrue(
            'tag_summary' in self.manifest['components']['report'],
            "No tag summary generated in report"
        )
        self.assertTrue(
            isinstance(
                self.manifest['components']['report']['tag_summary'],
                dict
            ),
            "Tag summary malformed"
        )

    def test_dir_listing_works(self):
        self.assertFileCollected('sos_commands/boot/ls_-alZR_.boot')
        boot_ls = self.get_name_in_archive('sos_commands/boot/ls_-alZR_.boot')
        with open(boot_ls, 'r', encoding='utf-8') as ls_file:
            # make sure we actually got ls output
            ln = ls_file.readline().strip()
            self.assertEqual(
                ln, '/boot:', f"dir listing first line looks incorrect: {ln}"
            )

    @redhat_only
    def test_version_matches_package(self):
        if not self.params.get('TESTLOCAL') == 'true':
            raise TestSkipError("Not testing local package installation")
        _pkg_ver = process.run('rpm -q sos').stdout.decode().split('-')[1]
        self.assertSosUILogContains(f"(version {_pkg_ver})")
        self.assertEqual(self.manifest['version'], _pkg_ver)


class LogLevelTest(StageOneReportTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '-vvv -o kernel,host,boot,filesys'

    def test_archive_logging_enabled(self):
        self.assertSosLogContains(r'DEBUG: \[archive:.*\]')
        self.assertSosLogContains('Making leading paths for')

    def test_debug_printed_to_console(self):
        self.assertOutputContains(r'\[plugin:.*\]')


class RestrictedSoSReport(StageOneReportTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = ('-o kernel,host,sudo,hardware,dbus,x11 --no-env-var '
               '--no-report -t1 --no-postproc')

    def test_no_env_vars_collected(self):
        self.assertFileNotCollected('environment')

    def test_no_reports_generated(self):
        self.assertFileNotCollected('sos_reports/sos.html')
        self.assertFileNotCollected('sos_reports/sos.json')
        self.assertFileNotCollected('sos_reports/sos.txt')

    def test_was_single_threaded_run(self):
        self.assertOutputNotContains('Finishing plugins')

    def test_postproc_not_called(self):
        self.assertOutputNotContains('substituting')

    def test_only_selected_plugins_run(self):
        self.assertOnlyPluginsIncluded(['kernel', 'host', 'sudo',
                                        'hardware', 'dbus', 'x11'])


class DisabledCollectionsReport(StageOneReportTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = ("-n networking,system,logs --skip-files=/etc/fstab "
               "--skip-commands='journalctl*'")

    def test_plugins_disabled(self):
        self.assertPluginNotIncluded('networking')
        self.assertPluginNotIncluded('system')
        self.assertPluginNotIncluded('logs')

    def test_skipped_plugins_have_no_dir(self):
        self.assertFileNotCollected('sos_commands/networking/')
        self.assertFileNotCollected('sos_commands/system/')
        self.assertFileNotCollected('sos_commands/logs/')

    def test_skip_files_working(self):
        self.assertFileNotCollected('/etc/fstab')

    def test_skip_commands_working(self):
        self.assertFileGlobNotInArchive('sos_commands/*/journalctl*')



# Copyright (C) Red Hat, Inc. 2019

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.policies.distros.redhat import RedHatPolicy


class AmazonPolicy(RedHatPolicy):
    vendor = "Amazon"
    vendor_urls = [('Distribution Website', 'https://aws.amazon.com')]
    os_release_file = ''
    os_release_name = 'Amazon Linux'

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Rhui(Plugin, RedHatPlugin):

    short_desc = 'Red Hat Update Infrastructure'

    plugin_name = "rhui"
    commands = ("rhui-manager", )
    files = ("/etc/rhui/rhui-tools.conf", )

    def setup(self):
        self.add_copy_spec([
            "/etc/rhui/rhui-tools.conf",
            "/etc/rhui/registered_subscriptions.conf",
            "/etc/pki/rhui/*",
            "/var/log/rhui-subscription-sync.log",
            "/var/cache/rhui/*",
            "/root/.rhui/*",
            "/var/log/rhui/*",
            "/var/log/rhui-installer/*",
        ])
        # skip collecting certificate keys
        self.add_forbidden_path("/etc/pki/rhui/**/*.key")

        # call rhui-manager commands with 1m timeout and
        # with an env. variable ensuring that "RHUI Username:"
        # even unanswered prompt gets collected
        self.add_cmd_output([
            "rhui-manager status",
            "rhui-manager cert info",
        ], timeout=60, env={'PYTHONUNBUFFERED': '1'})

        self.add_dir_listing('/var/lib/rhui/remote_share', recursive=True)

    def postproc(self):
        # hide rhui_manager_password value in (also rotated) answers file
        self.do_path_regex_sub(
                r"/root/\.rhui/answers.yaml.*",
                r"(\s*(rhui_manager|registry)_password\s*:)\s*(\S+)",
                r"\1********")
        # hide registry_password value in rhui-tools.conf
        self.do_path_regex_sub("/etc/rhui/rhui-tools.conf",
                               r"(registry_password:)\s*(.+)",
                               r"\1 ********")
        # obfuscate two cookies for login session
        for cookie in ["csrftoken", "sessionid"]:
            self.do_path_regex_sub(
                r"/root/\.rhui/.*/cookies.txt",
                fr"({cookie}\s+)(\S+)",
                r"\1********")


# vim: set et ts=4 sw=4 :
